#ifndef _MCONTROL_H
#include "melee/mcontrol.h"
#endif
#ifndef _MSHIP_H
#include "melee/mship.h"
#endif
#ifndef _MMAIN_H
#include "melee/mmain.h"
#endif

#ifndef _AIS_H
#define _AIS_H

//ais/c_input.cpp
class ControlKB : public Control {	
	private:
	int thrust;
	int left;
	int right;
	int fire;
	int special;
	int next;
	int prev;
	int close;
  public:
  virtual void load(char* inifile, char* inisection);
  virtual void save(char* inifile, char* inisection);
  virtual char *getTypeName();
  virtual int think();
	ControlKB (char *name, int id) ;
	virtual void setup();
	};
class ControlJoy : public Control {
	public:
  virtual void load(char* inifile, char* inisection);
  virtual void save(char* inifile, char* inisection);
  virtual void initialize();
  virtual void deinitialize();
	virtual char *getTypeName();
	virtual int think();
	ControlJoy (char *name, int id) ;
	virtual void setup();
	private:
	int which_joystick;
	int thrust;
	int left;
	int right;
	int fire;
	int special;
	int next;
	int prev;
	int close;
	};

//ais/c_other.cpp
int stupid_bot(Ship *ship) ;
class ControlVegetable : public Control {	
  public:
  virtual char *getTypeName();
  virtual int think();
	virtual int choose_ship(int player, int fleet_total);
	ControlVegetable (char *name, int id) ;
	};
class ControlMoron : public Control {	
  public:
  virtual char *getTypeName();
  virtual int think();
	ControlMoron (char *name, int id) ;
	};
class ControlRecord : public ControlWrapper {
	public:
	int f;
	ControlRecord (Control *who) ;
	virtual int think();
	virtual int choose_ship(int player, int fleet_total);
	virtual void initialize() ;
	virtual void deinitialize();
	virtual char *getTypeName();
	};
class ControlPlayback : public Control {
	public:
	virtual int choose_ship(int player, int fleet_total);
	virtual int think();
	ControlPlayback(char *name, int id);
	virtual char *getTypeName();
	};



//ais/c_wussie.cpp
class ControlWussie : public Control {
  public:
  virtual char *getTypeName();
  virtual int think();
	ControlWussie (char *name, int id) ;
	virtual void set_target(int i) ;
  virtual void select_ship(Ship* ship_pointer, char* ship_name);
	virtual double check_danger();
	double weapon_range;
	double weapon_velocity;
	int special_type;
	double special_freq;
	};

//ais/c_spy.cpp
class AI_Player;
class ControlSpyAI : public Control {	
	AI_Player *ai;
  public:
  virtual char *getTypeName();
  virtual int think();
  virtual void select_ship(Ship* ship_pointer, char* ship_name);
	ControlSpyAI (char *name, int id) ;
  virtual void deinitialize();
	};
class AI_Player;
class AI_Player_Earthling;
class AI_Player_Yehat;
class AI_Player_Pkunk;
class AI_Player_Utwig;
class AI_Player_Arilou;
class AI_Player_Slylandro;
class AI_Player_Chmmr;
//////////////////////////////
class KeyState {
  public:
  bool thrust;
  bool turn_left;
  bool turn_right;
  bool fire_weapon;
  bool fire_special;
  int convert();
};
class AI_Player {
  public:
//----------------------------------------------------------------------------//
   double debug_double[10]; // debug_info
//----------------------------------------------------------------------------//

   int  state;        // Current state of AI
   bool select_state; // Change or not current state of AI
   Ship *ai_ship;     // AI ship
	 KeyState k;

   double trajectory_angle(double x1, double y1, double x2, double y2);

   virtual void Stop(); //default 
   virtual void Chase(SpaceLocation *enemy); //default 
   virtual void Runaway(SpaceLocation *enemy, double range); //default 
   virtual void Turnface(SpaceLocation *enemy, double angle); //default 
   virtual void Attack(SpaceLocation *enemy); //default
//----------------------------------------------------------------------------//
   virtual void Attack_back(SpaceLocation *enemy);
//----------------------------------------------------------------------------//

   virtual void Evade(Shot *shot);       //default Evade from shot
   virtual void Evade(SpaceLine *laser); //default Evade from laser

   bool isTarget(Shot *shot);        // Shot will hit us?
   bool isTarget(SpaceLine *laser);  // Laser will hit us?

   bool nearPlanet();                // Planet near the us?

   void CheckShots(); // check all enemy shots and lasers for 'danger'

//------------------ subfunctions of CheckShots() ----------------------------//
//INFO: Differ names for no missunderstoods and correct calls in AIs
   virtual void ProcessShot(Shot *shot, double range);  
   virtual void ProcessLaser(SpaceLine *laser, double range); 
//----------------------------------------------------------------------------//

   virtual void calculate();
   virtual void change_state();

   AI_Player(Ship *ai);
};
//----------------------------------------------------------------------------//
// IMPLEMENTED AI CLASSES 
//----------------------------------------------------------------------------//
class AI_Player_Dumb : public AI_Player {

  public:
  AI_Player_Dumb(Ship *ai);

  virtual void ProcessShot(Shot *shot, double range);  
  virtual void ProcessLaser(SpaceLine *laser, double range); 
  virtual void calculate();
  virtual void change_state();
};

//----------------------------------------------------------------------------//
class AI_Player_Earthling : public AI_Player {
  protected:
   int close_table[20];

  public:
  AI_Player_Earthling(Ship *ai);

  virtual void ProcessShot(Shot *shot, double range);  
  virtual void ProcessLaser(SpaceLine *laser, double range); 
  virtual void calculate();
  virtual void change_state();
};

//----------------------------------------------------------------------------//
class AI_Player_Yehat : public AI_Player {
  public:
  AI_Player_Yehat(Ship *ai);

  virtual void ProcessShot(Shot *shot, double range);  
  virtual void ProcessLaser(SpaceLine *laser, double range); 
  virtual void calculate();
  virtual void change_state();
};

//----------------------------------------------------------------------------//
class AI_Player_Utwig : public AI_Player {
  public:
  AI_Player_Utwig(Ship *ai);

  virtual void ProcessShot(Shot *shot, double range); 
  virtual void ProcessLaser(SpaceLine *laser, double range); 
  virtual void calculate();
  virtual void change_state();
};

//----------------------------------------------------------------------------//
class AI_Player_Pkunk : public AI_Player {
  protected:
   int evade_table[20];

  public:
  AI_Player_Pkunk(Ship *ai);

  virtual void Attack(SpaceLocation *enemy);
  virtual void ProcessShot(Shot *shot, double range);  
  virtual void ProcessLaser(SpaceLine *laser, double range); 
  virtual void calculate();
  virtual void change_state();
};

//----------------------------------------------------------------------------//
class AI_Player_Arilou : public AI_Player {
  public:
  AI_Player_Arilou(Ship *ai);

  virtual void ProcessShot(Shot *shot, double range);  
  virtual void ProcessLaser(SpaceLine *laser, double range); 
  virtual void calculate();
  virtual void change_state();
};

//----------------------------------------------------------------------------//
class AI_Player_Slylandro : public AI_Player {
  public:
  AI_Player_Slylandro(Ship *ai);

  virtual void ProcessShot(Shot *shot, double range);  
  virtual void ProcessLaser(SpaceLine *laser, double range); 
  virtual void calculate();
  virtual void change_state();
};

//----------------------------------------------------------------------------//
class AI_Player_Chmmr : public AI_Player {
  public:
  AI_Player_Chmmr(Ship *ai);
  bool TractorBeam;

  virtual void calculate();
  virtual void change_state();
};

//----------------------------------------------------------------------------//
class AI_Player_Druuge : public AI_Player {
  public:
  AI_Player_Druuge(Ship *ai);

  virtual void ProcessShot(Shot *shot, double range);  
  virtual void ProcessLaser(SpaceLine *laser, double range); 
  virtual void calculate();
  virtual void change_state();
};
//----------------------------------------------------------------------------//
class AI_Player_Kohrah : public AI_Player {
  public:
  AI_Player_Kohrah(Ship *ai);

  double old_range;
  bool Attack_Cancel;

  virtual void Attack(SpaceLocation *enemy);
  virtual void ProcessShot(Shot *shot, double range);  
  virtual void ProcessLaser(SpaceLine *laser, double range); 
  virtual void calculate();
  virtual void change_state();
};

//----------------------------------------------------------------------------//
class AI_Player_Chenjesu : public AI_Player {
  public:
  AI_Player_Chenjesu(Ship *ai);

  double old_range;
  bool Attack_Cancel;

  virtual void Attack(SpaceLocation *enemy);
  virtual void ProcessShot(Shot *shot, double range);  
  virtual void ProcessLaser(SpaceLine *laser, double range); 
  virtual void calculate();
  virtual void change_state();
};
//----------------------------------------------------------------------------//
#endif
