
#include <string.h>
#include "../ais.h"
#include "../melee/mshot.h"
#include "../melee/mcbodies.h"

//----------------------------------------------------------------------------//
// INITIALIZING AI 
AI_Player *init_ai(Ship *ai, char *ai_ID)
{
	if (!ai_ID) return NULL;
  if (!strcmp(ai_ID, "arisk")) return new AI_Player_Arilou(ai);
  if (!strcmp(ai_ID, "chmav")) return new AI_Player_Chmmr(ai);
  if (!strcmp(ai_ID, "earcr")) return new AI_Player_Earthling(ai);
  if (!strcmp(ai_ID, "pkufu")) return new AI_Player_Pkunk(ai);
  if (!strcmp(ai_ID, "yehte")) return new AI_Player_Yehat(ai);
  if (!strcmp(ai_ID, "utwju")) return new AI_Player_Utwig(ai);
  if (!strcmp(ai_ID, "slypr")) return new AI_Player_Slylandro(ai);

  if (!strcmp(ai_ID, "druma")) return new AI_Player_Druuge(ai);
  if (!strcmp(ai_ID, "kohma")) return new AI_Player_Kohrah(ai);
  if (!strcmp(ai_ID, "chebr")) return new AI_Player_Chenjesu(ai);
  
 return new AI_Player_Dumb(ai);
}

void ControlSpyAI::select_ship(Ship* ship_pointer, char* ship_name) {
//	if (ai) delete ai;
	ai = init_ai(ship_pointer, ship_name);
	Control::select_ship(ship_pointer, ship_name);
	return;
	}
int ControlSpyAI::think() {
	if (ai) {
		ai->calculate();
		return ai->k.convert();
		}
	return stupid_bot(ship);
	}
void ControlSpyAI::deinitialize() {
//	if (ai) delete ai;
	Control::deinitialize();
	return;
	}
ControlSpyAI::ControlSpyAI(char *name, int id) : Control(name, id) {
	ai = NULL;
	return;
	}
char *ControlSpyAI::getTypeName() {
	return "SpyAI";
	}
//----------------------------------------------------------------------------//
// BASE AI CLASS 
AI_Player::AI_Player(Ship *ai)
{
      ai_ship = ai;
 select_state = true;
}
//----------------------------------------------------------------------------//
// tween of trajectrory but differ args 
double AI_Player::trajectory_angle(double x1, double y1, double x2, double y2)
{ 
  double dx = min_delta(x1, x2, X_MAX);
  double dy = min_delta(y1, y2, Y_MAX);
  double alpha = atan(dy / dx);

  if(dx < 0.0)
    alpha += PI;
  alpha = normalize(alpha, PI * 2.0);

  alpha = ((alpha + PI) / (PI * 2.0)) * 360.0;
  alpha = normalize(alpha, 360.0);
 return alpha;
}
//----------------------------------------------------------------------------//
// Slowdown ship 
// INFO: Seems work ideal ;) 
void AI_Player::Stop()
{
 k.turn_left  = false;
 k.turn_right = false;
 k.thrust     = false;

 double     ai_x = ai_ship->normal_x();
 double     ai_y = ai_ship->normal_y();
 double    ai_vx = ai_ship->get_vx(); 
 double    ai_vy = ai_ship->get_vy(); 

 if (sqrt(ai_vx*ai_vx+ai_vy*ai_vy)<0.10)  // speed is very low = stop
  {
   select_state = true; //FINISH 'STOP'
   return;
  }
 double alpha = trajectory_angle(ai_x, ai_y, ai_x+ai_vx, ai_y+ai_vy); // get angle of direction
        alpha = ai_ship->get_angle() - alpha;

  if (alpha < 0.0) alpha += 360.0;

  if (alpha < 180.0 - 5.6251) 
   k.turn_right = true; 
  else
  if (alpha > 180.0 + 5.6251) 
   k.turn_left  = true;
  else
   k.thrust = true;
}
//----------------------------------------------------------------------------//
// Chase enemy 
// INFO: Seems work good ;) 
void AI_Player::Chase(SpaceLocation *enemy)
{
 k.turn_left    = false;
 k.turn_right   = false;
 k.thrust       = true;

 double  a_x = ai_ship->normal_x(); 
 double  a_y = ai_ship->normal_y();
 double a_vx = ai_ship->get_vx(); 
 double a_vy = ai_ship->get_vy();
 double  e_x = enemy->normal_x(); 
 double  e_y = enemy->normal_y();
 double e_vx = enemy->get_vx();
 double e_vy = enemy->get_vy();

// double alpha = ai_ship->get_angle() - enemy->trajectory_angle(ai_ship); 
 double alpha = ai_ship->get_angle() - trajectory_angle(e_x + e_vx, e_y + e_vy, 
                                                        a_x + a_vx, a_y + a_vy);

 if (alpha < 0.0) alpha += 360.0;

 if (alpha < 180.0 - 5.6251) 
  k.turn_right = true; 
 else
  if (alpha > 180.0 + 5.6251) 
   k.turn_left  = true;
 else
   select_state=true;
}
//----------------------------------------------------------------------------//
// Runaway from enemy 
// INFO: Seems work good ;) 
void AI_Player::Runaway(SpaceLocation *enemy, double range)
{
 k.turn_left  = false;
 k.turn_right = false;
 k.thrust     = true;

 double    ai_angle = ai_ship->get_angle(); 
 double enemy_angle = enemy->trajectory_angle(ai_ship);

 if (ai_angle < enemy_angle) k.turn_right = true;
 if (ai_angle > enemy_angle) k.turn_left  = true;

 if (ai_ship->distance(enemy)>range) 
  select_state=true;
}
//----------------------------------------------------------------------------//
// Turn face to enemy for 'angle' 
// INFO: Seems work good ;) 
void AI_Player::Turnface(SpaceLocation *enemy, double angle)
{
 k.turn_left    = false;
 k.turn_right   = false;
 double alpha = ai_ship->get_angle() - enemy->trajectory_angle(ai_ship); 
 if (alpha < 0.0) alpha += 360.0;

 if (alpha < 180.0 - 5.6251 - angle) 
  k.turn_right = true; 
 else
  if (alpha > 180.0 + 5.6251 + angle) 
   k.turn_left  = true;
 else
   select_state=true;
}
//----------------------------------------------------------------------------//
// ATTACK
void AI_Player::Attack(SpaceLocation *enemy)
{
 k.turn_left  = false;
 k.turn_right = false;

 double  a_x = ai_ship->normal_x(); 
 double  a_y = ai_ship->normal_y();
 double a_vx = ai_ship->get_vx(); 
 double a_vy = ai_ship->get_vy();
 double  e_x = enemy->normal_x(); 
 double  e_y = enemy->normal_y();
 double e_vx = enemy->get_vx();
 double e_vy = enemy->get_vy();

 // targeting for future position is still not perfect :((((
// double alpha = ai_ship->get_angle() - trajectory_angle(e_x + e_vx, e_y + e_vy, 
//                                                        a_x + a_vx, a_y + a_vy);

 double alpha = ai_ship->get_angle() - trajectory_angle(e_x + e_vx, e_y + e_vy, 
                                                        a_x + a_vx, a_y + a_vy);

 if (alpha < 0.0) alpha += 360.0;

 if (alpha < 180.0 - 5.6251) 
  k.turn_right = true; 
 else
  if (alpha > 180.0 + 5.6251) 
   k.turn_left  = true;
 else
  { 
    select_state=true;
    k.fire_weapon = true;
  }
}  
//----------------------------------------------------------------------------//
// ENEMY SHOT IS TARGETED TO US?
// INFO: Seems work good. But not ideal ;)
bool AI_Player::isTarget(Shot *shot)
{
  double  ai_x = ai_ship->normal_x();
  double  ai_y = ai_ship->normal_y();
  double ai_vx = ai_ship->get_vx(); 
  double ai_vy = ai_ship->get_vy(); 

  double  shot_x = shot->normal_x();
  double  shot_y = shot->normal_y();
  double shot_vx = shot->get_vx(); 
  double shot_vy = shot->get_vy(); 

  double      range = ai_ship->distance(shot);
  double next_range = distance_from(ai_x + ai_vx, ai_y + ai_vy, 
                       shot_x + shot_vx, shot_y + shot_vy);

 if (next_range >= range) return false; //differ directions 

 double range_step = range - next_range;
 
    double k = range / range_step;
  next_range = distance_from(ai_x + ai_vx*k, ai_y + ai_vy*k, 
                shot_x + shot_vx*k, shot_y + shot_vy*k);

  if (next_range<=50.0) // in ideal...there must be 'ship & shot width' factor  
   return true;         // but not only range between 2 points!!!!
  else
   return false;
}
//----------------------------------------------------------------------------//
// ENEMY LASER IS TARGETED TO US?
// INFO: Seems work normal. But detect too close.
bool AI_Player::isTarget(SpaceLine *laser)
{
  double  ai_x = ai_ship->normal_x()+ai_ship->get_vx();
  double  ai_y = ai_ship->normal_y()+ai_ship->get_vy();

  double  laser_x = laser->normal_x()+laser->edge_x();
  double  laser_y = laser->normal_y()+laser->edge_y();

  double      range = ai_ship->distance(laser);
  double next_range = distance_from(ai_x, ai_y, laser_x, laser_y);

 if (next_range >= range) return false; //differ directions 

  if (next_range<=50.0) // in ideal...there must be 'ship width' factor  
   return true;         // but not only range between 2 points!!!!
  else
   return false;
}
//----------------------------------------------------------------------------//
// Trying to EVADE from enemy shot 
void AI_Player::Evade(Shot *shot)
{

 select_state = true;

 if (!isTarget(shot)) return; // We're targeted?

 k.turn_left  = false;
 k.turn_right = false;
 k.thrust     = true;

 double x1 = ai_ship->normal_x(); 
 double y1 = ai_ship->normal_y(); 
 double x2 = shot->normal_x();
 double y2 = shot->normal_y();

 bool left=false, up=false; 

 if (x1 <= x2) left = true;
 if (y1 <= y2)   up = true;

 if (left && up) 
  k.turn_right = true; 
 else
  k.turn_left  = true;
}

//----------------------------------------------------------------------------//
// Trying to EVADE from enemy laser 
// INFO: It's simple copy of 'Evade from shot. But it must be differ!!!
void AI_Player::Evade(SpaceLine *laser)
{

 select_state = true;

 if (!isTarget(laser)) return; // We're targeted?

 k.turn_left  = false;
 k.turn_right = false;
 k.thrust     = true;

 double x1 = ai_ship->normal_x(); 
 double y1 = ai_ship->normal_y(); 
 double x2 = laser->normal_x();
 double y2 = laser->normal_y();

 bool left=false, up=false; 

 if (x1 <= x2) left = true;
 if (y1 <= y2)   up = true;

 if (left && up) 
  k.turn_right = true; 
 else
  k.turn_left  = true;
}

//----------------------------------------------------------------------------//
bool AI_Player::nearPlanet()
{
 if (ai_ship->distance(spacePlanet)<500.0) 
  return true;
 else
 return false;
}
//----------------------------------------------------------------------------//
// simple check all enemy shots and lasers for 'danger' 
void AI_Player::CheckShots()
{
 Shot *shot;
 shot = (Shot*)spaceFrame->getFirstObject(LAYER_SHOTS);
 double shot_range;

 while (shot!=NULL)
 {
  if (!ai_ship->isAlly(shot)) {
  shot_range = ai_ship->distance(shot); 
   ProcessShot(shot, shot_range);          //process this shot
  }
  shot=(Shot*)shot->next;
 }

 SpaceLine *Laser;
 Laser = spaceFrame->getFirstLine();
 while (Laser!=NULL)
 {
  if (!ai_ship->isAlly(Laser)) {

   shot_range = ai_ship->distance(Laser); 
   ProcessLaser(Laser, fabs(shot_range-Laser->get_length())); //process this laser
  }
  Laser=Laser->nextl;
 }

}
//----------------------------------------------------------------------------//
// what we must do with 'danger shot'
void AI_Player::ProcessShot(Shot *shot, double range)
{
}
//----------------------------------------------------------------------------//
// what we must do with 'danger laser'
void AI_Player::ProcessLaser(SpaceLine *shot, double range)
{
}

//----------------------------------------------------------------------------//
void AI_Player::calculate()
{
 k.turn_left    = false;
 k.turn_right   = false;
 k.fire_weapon  = false;
 k.fire_special = false;
 k.thrust       = false;
}

//----------------------------------------------------------------------------//
void AI_Player::change_state()
{
}
//----------------------------------------------------------------------------//
// ATTACK
void AI_Player::Attack_back(SpaceLocation *enemy)
{
 k.turn_left  = false;
 k.turn_right = false;

 double  a_x = ai_ship->normal_x(); 
 double  a_y = ai_ship->normal_y();
 double a_vx = ai_ship->get_vx(); 
 double a_vy = ai_ship->get_vy();
 double  e_x = enemy->normal_x(); 
 double  e_y = enemy->normal_y();
 double e_vx = enemy->get_vx();
 double e_vy = enemy->get_vy();

 double alpha = ai_ship->get_angle() - ai_ship->target->get_angle();
// double alpha = ai_ship->target->get_angle() - ai_ship->get_angle();

 if (alpha < 0.0) alpha += 360.0;

 if (alpha < 180.0 - 5.6251) 
   k.turn_left  = true;
 else
  if (alpha > 180.0 + 5.6251) 
   k.turn_right = true; 
 else
  { 
    select_state = true;
//    k.fire_weapon = true;
  }
}

int KeyState::convert() {
  int a;
  a = 0;
  if (thrust)       a |= keyflag::thrust;
  if (turn_left)    a |= keyflag::left;
  if (turn_right)   a |= keyflag::right;
  if (fire_weapon)  a |= keyflag::fire;
  if (fire_special) a |= keyflag::special;
  return a;
}

