#include <string.h>

#include "../melee.h"
#include "../ais.h"
#include "../melee/mcontrol.h"
#include "../melee/mframe.h"
#include "../melee/mship.h"
#include "../melee/mmain.h"

#define SPECIAL_UNKNOWN 0
#define SPECIAL_DEFENSE 1
#define SPECIAL_BATTERY 2
#define SPECIAL_NONE    3

double ControlWussie::check_danger() {
	double d = 0;
	SpaceObject *o = spaceFrame->getFirstObject(LAYER_SHOTS);
	while (o) {
		if ((ship->distance(o) < (ship->w+ship->h+o->w+o->h+5)) && (o->damage_factor > 0)) {
			double  ai_x = ship->normal_x();
			double  ai_y = ship->normal_y();
			double ai_vx = ship->get_vx(); 
			double ai_vy = ship->get_vy(); 

			double  enemy_x = o->normal_x();
			double  enemy_y = o->normal_y();
			double enemy_vx = o->get_vx(); 
			double enemy_vy = o->get_vy(); 

			double      range = ship->distance(o);
			double next_range = distance_from(ai_x + ai_vx, ai_y + ai_vy, 
					enemy_x + enemy_vx, enemy_y + enemy_vy);

			if (next_range < range) d += o->damage_factor;

//			double range_step = range - next_range;
			}
		o = o->nexto;
		}
	return d;
	}

char *ControlWussie::getTypeName() {
	return "WussieBot";
	}
int ControlWussie::think() {
	int action = 0;
	if (!ship->target) return 0;
	if (!ship->target->exists()) {
		ship->target = NULL;
		return 0;
		}
	float distance = ship->distance(ship->target);
	float velocity;
	if (distance < weapon_range) velocity = weapon_velocity;
	else velocity = ship->speed_max;

	double rx, ry;
	if (velocity == 0) velocity = GLOBAL_MAXSPEED;
	rx = -ship->normal_x() + ship->target->normal_x() + (ship->target->get_vx()) * distance / velocity;
	rx = normalize(rx, X_MAX);
	if (rx > X_MAX / 2) rx -= X_MAX;
	ry = -ship->normal_y() + ship->target->normal_y() + (ship->target->get_vy()) * distance / velocity;
	ry = normalize(ry, Y_MAX);
	if (ry > Y_MAX / 2) ry -= Y_MAX;
	double angle = atan2(ry, rx) * 180 / PI - ship->get_angle();
	while (angle < 0) angle += 360;
	while (angle > 360) angle -= 360;
	angle -= 180;
	if (angle > 0) {
		action |= keyflag::left;
		}
	else {
		action |= keyflag::right;
		}
	action |= keyflag::thrust;
  if ((distance < weapon_range) && (abs(angle < 30))) {
		action |= keyflag::fire;
    }
	switch (special_type) {
		default:
		case SPECIAL_UNKNOWN:
			if (ship->batt == ship->batt_max) action |= keyflag::special;
			break;
		case SPECIAL_DEFENSE:
			if (check_danger() > 0) action |= keyflag::special;
			break;
		case SPECIAL_BATTERY:
			if (ship->batt < ship->batt_max) action |= keyflag::special;
			break;
		case SPECIAL_NONE:
			break;
		}
	if ((rand() % 4000) < frame_time) action |= keyflag::close;
  return action;
	}

ControlWussie::ControlWussie(char *name, int id) : Control(name, id) {}

void ControlWussie::set_target(int i) {
	Control::set_target(i);
	}
void ControlWussie::select_ship(Ship* ship_pointer, char* ship_name) {
	char tmp[20];
	Control::select_ship(ship_pointer, ship_name);
	if (ship_name) {
		replace_extension(tmp, ship_name, "ini", 19);
		weapon_range = scale_range(get_config_float("Weapon", "Range", 50));
		weapon_velocity = scale_velocity(get_config_float("Weapon", "Velocity", 999));
		special_freq = get_config_float ("AI3_Default", "SpecialFreq", 1);
		const char *s = get_config_string ("AI3_Default", "Special", "");
		special_type = SPECIAL_UNKNOWN;
		if      (!strcmp ( s, "Defense")) special_type = SPECIAL_DEFENSE;
		else if (!strcmp ( s, "Battery")) special_type = SPECIAL_BATTERY;
		else if (!strcmp ( s, "None")) special_type = SPECIAL_NONE;
		}
	return;
	}

