
#include <stdlib.h>

#include "../melee.h"
#include "../id.h"
#include "../scp.h"

#include "mview.h"
#include "mcbodies.h"
#include "mmain.h"
#include "manim.h"
#include "mship.h"


// Define global asteroid sprites
SpaceSprite *asteroidSprite;
SpaceSprite *asteroidExplosionSprite;

Asteroid::Asteroid() :
  SpaceObject(0.0, 0.0, 0.0, -1, asteroidSprite)
{
  collide_flag_anyone = 0xff;
	collide_flag_ally = 0xff;
	layer = LAYER_CBODIES;
  id    = SPACE_ASTEROID;
  step  = 0;
  speed = ((random() % 3) + 1);
  mass  = 10 + (random() % 10);

  radar_color = RADAR_COLOR_ASTEROID;

  // Angles are clockwise, turned left at zero degrees
  switch(random() % 4) {
    case 0:
      // Entering from upper portion
      x = (double)(random() % (int)(X_MAX));
      y = 0.0;

      // Between 45 to 135 degrees
      angle = (double)(random() % 90) + 45.0;
      break;

    case 1:
      // Entering from lower portion
      x = (double)(random() % (int)(X_MAX));
      y = Y_MAX;

      // Between 225 to 315 degrees
      angle = (double)(random() % 90) + 225.0;
      break;

    case 2:
      // Entering from left portion
      x = 0.0;
      y = (double)(random() % (int)(Y_MAX));

      // Between -45 to 45 degrees
      angle = (double)(random() % 90) - 45.0;
      break;

    case 3:
      // Entering from right portion
      x = X_MAX;
      y = (double)(random() % (int)(Y_MAX));

      // Between 125 to 215 degrees
      angle = (double)(random() % 90) + 125.0;
      break;
  }

  while(angle < 0.0)
    angle += 360.0;
  while(angle > 360.0)
    angle -= 360.0;

  vx = speed * cos(angle * ANGLE_RATIO) * 0.05;
  vy = speed * sin(angle * ANGLE_RATIO) * 0.05;

}

void Asteroid::calculate()
{
  while(step <= 0) {
    step += speed * time_ratio;
    sprite_index++;
    if(sprite_index == ASTEROID_FRAMES)
      sprite_index = 0;
  }
  step-= frame_time;

  SpaceObject::calculate();
}

void Asteroid::handle_damage(SpaceLocation *source) {
	if ((damage == 0) && (directDamage == 0)) return;

	damage = 0;
	directDamage = 0;
	spaceFrame->addSpaceObject(new Animation(x, y,
			asteroidExplosionSprite, ASTEROIDEXPLOSION_FRAMES, time_ratio, LAYER_CBODIES));

	x = (double)(random() % (int)(X_MAX));
	y = (double)(random() % (int)(Y_MAX));
	speed = ((random() % 3) + 1);
	mass  = 10 + (random() % 5);
	angle = (double)(random() % 360);
	vx = speed * cos(angle * ANGLE_RATIO) * 0.05;
	vy = speed * sin(angle * ANGLE_RATIO) * 0.05;
	}

Planet::Planet(SpaceSprite *sprite) :
		SpaceObject(X_MAX / 2.0, Y_MAX / 2.0, 0.0, -1, sprite) {
	collide_flag_anyone = 0xff;
	collide_flag_ally = 0xff;
	layer = LAYER_CBODIES;
	id          = SPACE_PLANET;
	mass        = 99999.0;
	radar_color = RADAR_COLOR_PLANET;
//use remote .ini file
	set_config_file ("scp.ini");
	gravity_mindist = scale_range(get_config_float("Planet", "GravityMinDist", 0));
	gravity_range = scale_range(get_config_float("Planet", "GravityRange", 0));
	gravity_power = get_config_float("Planet", "GravityPower", 0);
	gravity_force = 
		scale_acceleration(get_config_float("Planet", "GravityForce", 0), 0) *
		pow ( 200, gravity_power);;
//global, not planetary variable
	gravity_sling = get_config_float("Planet", "GravitySling", 0);
	}

void Planet::inflict_damage(SpaceObject *other) {
	int i = 1;
	if (other->isShip()) {
		i = (int) ceil(((Ship*)other)->crew / 3.0);
		}
	if (other->mass == 0) other->state = 0;
	other->directDamage += i;
	i /= 2;
	if (i >= BOOM_SAMPLES) i = BOOM_SAMPLES - 1;
	if (!other->isShot()) 
		tw_sample((SAMPLE *)(melee[MELEE_BOOM + i].dat));
	other->handle_damage(this);
	}

void Planet::calculate() {
	SpaceObject::calculate();
	SpaceObject *o;
	Query a;
	a.begin(this, bit(SPACE_LAYERS)-1, gravity_range);
	for (;a.currento;a.next()) {
		o = a.currento;
		if (o->mass > 0) {
			double r = distance(o);
			if (r < gravity_mindist) r = gravity_mindist;
			double sr = 1;
//gravity_power truncated here
			for (int p = (int)gravity_power; p > 0; p -= 1) {
				sr *= r;
				}
			o->accelerate(this, trajectory_angle(o) + 180, (double)frame_time * gravity_force / sr, GLOBAL_MAXSPEED);
			}
		}
	return;
	}
