#ifndef _MELEE_H
#include "../melee.h"
#endif

#ifndef _MCONTROL_H
#define _MCONTROL_H

extern const char num_controls;
extern char **control_name;
Control *getController(int type, char *name, int id);
int control2number(const char *name) ;

class Control {	
	public:
	int id;
//player ID number
	Ship *ship;
//points at the current ship being controlled
	char *iname;
//name of instance
	int num_targets;
//number of targets
	Ship **target;
//array of targets
	int active_target;
//index (into the array, target) of the currently active target)
	int always_random;
//always-random selection, for the always-random button on the default ship selector
	public:
	virtual void load(char* inifile, char* inisection);
//loads configuration data from a ini file. 
//example:  player1.load("scp.ini", "Keyboard1");
	virtual void save(char* inifile, char* inisection);
//saves configuration data to a ini file. 
	virtual int choose_ship(int player, int fleet_total);
//presents the ship selection dialog
	virtual void select_ship(Ship* ship_pointer, char* ship_name);
//called whenever the ship being controlled changes
	void calculate();
//called every frame of physics
	virtual int think();
//called by calculate... this is where the important stuff goes
	virtual void initialize() ;
//called whenever a new melee is started
	virtual void deinitialize();
//called whenever the current melee ends
	virtual char *getTypeName() = 0;
//returns the name of the control type, like "Joystick"
	virtual char *getDescription();
//returns the description of the control, like "NetworkGame(local)_Keyboard0"
	Control (char *name, int id) ;
//the constructor initializes ship to NULL
	virtual ~Control () ;
//the destructor, which is rarely used
	virtual void setup();
//called by the GUI stuff to setup the controls (calibrate Joystick, set keys, etc.)
	virtual void add_target(Ship *killit) ;
	virtual void remove_target(Ship *killit) ;
	virtual void set_target(int i) ;
//sets the target
	};

class ControlWrapper : public Control {
	public:
	Control *c;
	int created;
	virtual void load(char* inifile, char* inisection);
	virtual void save(char* inifile, char* inisection);
	virtual int choose_ship(int player, int fleet_total);
	virtual void select_ship(Ship* ship_pointer, char* ship_name);
	void calculate();
	virtual int think();
	virtual void initialize() ;
	virtual void deinitialize();
	virtual char *getTypeName();
	virtual char *getDescription();
	ControlWrapper (char *name, int id) ;
	virtual ~ControlWrapper () ;
	virtual void setup();
	virtual void add_target(Ship *killit) ;
	virtual void remove_target(Ship *killit) ;
	virtual void set_target(int i) ;
	};



#endif