
#include <allegro.h>
#include "../melee.h"
#include "../libs.h"
#include "../id.h"
#include "../frame.h"
#include "mmain.h"
#include "mframe.h"
#include "manim.h"
#include "mview.h"

const int DEATH_FRAMES = 3;

#define QUADS_X 16
#define QUADS_Y 16
#define QUADS_TOTAL (QUADS_X * QUADS_Y)
#define QUAD_X (X_MAX / QUADS_X)
#define QUAD_Y (Y_MAX / QUADS_Y)
#define QUADI_X (1.0 / QUAD_X)
#define QUADI_Y (1.0 / QUAD_Y)

void Query_old::begin(SpaceLocation *qtarget, int qlayers, double qrange) {
	layers = qlayers;
	range = qrange;
	target = qtarget;
	currento = NULL;
	layer = -1;
	next_layer();
	if (!current) return;
	if ((target->distance(currento) > range) || (currento == target)) next();
	return;
	}

void Query_old::next_layer () {
tail_recurse:
	if (layer >= SPACE_LAYERS) {
		currento = NULL;
		return;
		}
	layer += 1;
	if (!(bit(layer) & layers)) goto tail_recurse;
	if (layer != LAYER_LINES) currento = spaceFrame->getFirstObject(layer);
//	else current = spaceFrame->getFirstLine();
	if (!currento) goto tail_recurse;
	return;
	}

void Query_old::next() {
tail_recurse2:
	currento = currento->nexto;
	if (!currento) {
		next_layer();
		if (currento && (target->distance(currento) > range)) goto tail_recurse2;
		return;
		}
	if (target->distance(currento) > range) goto tail_recurse2;
	if (currento == target) goto tail_recurse2;
	return;
	}

void Query_new::begin (SpaceLocation *qtarget, int qlayers, double qrange) {
	layers = qlayers;
	range = qrange;
	target = qtarget;
	currento = NULL;
	if (target->isObject()) range += ((SpaceObject*)target)->w / 2;
	qx_min = (int)floor((target->normal_x() - range - 128) * QUADI_X);
	qx_max = int((target->normal_x() + range + 128) * QUADI_X + 1);
	if (qx_min < 0) qx_min += QUADS_X;
	if (qx_max >= QUADS_X) qx_max -= QUADS_X;
	qy_min = (int)floor((target->normal_y() - range - 128) * QUADI_Y);
	qy_max = int((target->normal_y() + range + 128) * QUADI_Y + 1);
	if (qy_min < 0) qy_min += QUADS_Y;
	if (qy_max >= QUADS_Y) qy_max -= QUADS_Y;
	qy = qy_min;
	qx = qx_min - 1;
	next_quadrant();
	if (!current) return;
	if ((target->distance(current) > range) || (current == target) || 
			!(bit(current->layer) & layers) || !current->exists()) {
		next();
		return;
		}
	return;
	}

void Query_new::next_quadrant () {
tail_recurse4:
	qx += 1;
	if (qx == QUADS_X) qx = 0;
	if (qx == qx_max) {
		qy += 1;
		if (qy == QUADS_Y) qy = 0;
		if (qy == qy_max) {
			current = NULL;
			qy -= 1;
			qx -= 1;
			return;
			}
		qx = qx_min;
		}
	current = spaceFrame->quadranto[qy * QUADS_X + qx];
	if (!current) goto tail_recurse4;
	return;
	}

void Query_new::next () {
tail_recurse3:
	current = current->qnext;
	if (!current) {
		next_quadrant();
		if (!current) return;
		}
	if (target->distance(current) > range) goto tail_recurse3;
	if (current == target) goto tail_recurse3;		
	if (!(bit(current->layer) & layers)) goto tail_recurse3;
	if (!current->exists()) goto tail_recurse3;
	return;
	}
void Query_new::end() {
	}


SpaceLocation::SpaceLocation(double lx, double ly, double langle, int team) :
  id(SPACE_LOCATION),
  x(lx),
  y(ly),
  angle(langle),
  vx(0.0),
  vy(0.0),
  state(1),
	layer(0),
  damage_factor(0),
  collide_flag_anyone(bit(SPACE_LAYERS) - 1),
  collide_flag_ally(0),
  ally_flag(team),
  damage(0),
  directDamage(0),
  repair(0),
  fuel_sap(0),
  speed_loss(0.0), 
	next(NULL),
	qnext(NULL)
{
}

void SpaceLocation::death() {
	}

double SpaceLocation::get_angle()
{
	return(normalize(angle, 360.0));
}

double SpaceLocation::get_vx()
{
  return(vx);
}

double SpaceLocation::get_vy()
{
  return(vy);
}

int SpaceLocation::getID()
{
  return id;
}

int SpaceLocation::allyFlag()
{
  return ally_flag;
}

double SpaceLocation::normal_x()
{
  return(normalize(x, X_MAX));
}

double SpaceLocation::normal_y()
{
  return(normalize(y, Y_MAX));
}

double SpaceLocation::nearest_x(SpaceLocation *l)
{
  return(nearest_coord(normal_x(), l->normal_x(), X_MAX));
}

double SpaceLocation::nearest_y(SpaceLocation *l)
{
  return(nearest_coord(normal_y(), l->normal_y(), Y_MAX));
}

double SpaceLocation::distance(SpaceLocation *l)
{
  return(distance_from(normal_x(), normal_y(), l->normal_x(), l->normal_y()));
}

double SpaceLocation::trajectory_angle(SpaceLocation *l)
{
	double dx = min_delta(l->normal_x(), normal_x(), X_MAX);
  double dy = min_delta(l->normal_y(), normal_y(), Y_MAX);

	double alpha = atan2(dy, dx) * (180 / PI);
  alpha = normalize(alpha, 360.0);
  return(alpha);
}

int SpaceLocation::exists()
{
  return (state > 0);
}

 int SpaceLocation::canCollide(SpaceLocation *other)
{
  if(other == NULL)
    return(FALSE);

	if (ally_flag == other->allyFlag()) return ((1 << other->layer) & collide_flag_ally);
	else return ((1 << other->layer) & collide_flag_anyone);
}

int SpaceLocation::isAlly(SpaceLocation *other)
{
  return(ally_flag == other->ally_flag);
}

int SpaceLocation::isLine() {
	return ((id & BASE_MASK) == SPACE_LINE);
	}

int SpaceLocation::isObject() {
	return ((id & BASE_MASK) == SPACE_OBJECT);
	}

int SpaceLocation::isPlanet() {
	return (id == SPACE_PLANET);
	}

int SpaceLocation::isAsteroid() {
	return (id == SPACE_ASTEROID);
	}

int SpaceLocation::isShip() {
	return ((id & SPACE_MASK) == SPACE_SHIP);
	}

int SpaceLocation::isShot() {
	return ((id & SPACE_MASK) == SPACE_SHOT);
	}

int SpaceLocation::translate( double dx, double dy) {
	x = normalize (x + dx, X_MAX);
	y = normalize (y + dy, Y_MAX);
	return true;
	}

int SpaceLocation::accelerate(SpaceLocation *source, double angle, double velocity, double max_speed) {
	double ov, nv;
	double nvx, nvy;

	ov = (vx * vx) + (vy * vy);
	nvx = vx + (cos(angle * ANGLE_RATIO) * velocity);
	nvy = vy + (sin(angle * ANGLE_RATIO) * velocity);

	nv = (nvx * nvx) + (nvy * nvy);

	if ((nv <= max_speed * max_speed) || (nv <= ov)) {
		vx = nvx;
		vy = nvy;
		}
	else {
		if (ov <= max_speed * max_speed) ov = max_speed;
		else ov = sqrt(ov);
		vx = nvx * ov / (ov + velocity);
		vy = nvy * ov / (ov + velocity);
		}
	return true;
	}

void SpaceLocation::animate(Frame *space) {
	return;
	}
void SpaceLocation::calculate() {
	return;
	}

SpaceObject::SpaceObject(double ox, double oy, double oangle, int team,
  SpaceSprite *osprite) :
  SpaceLocation(ox, oy, oangle, team),
  w(osprite->width()),
  h(osprite->height()),
  mass(0),
  sprite(osprite),
  sprite_index(0),
  radar_color(-1)
{
		
  id = SPACE_OBJECT;
	collide_flag_anyone = bit(SPACE_LAYERS) - 1;
	collide_flag_ally = 0;
}

double SpaceObject::corner_x()
{
  return(wcorner(x, w));
}

double SpaceObject::corner_y()
{
  return(hcorner(y, h));
}

int SpaceObject::getSpriteIndex()
{
  return(sprite_index);
}
 
SpaceSprite* SpaceObject::getSprite()
{
  return( sprite );
}

double SpaceObject::width()
{
  return(sprite->width());
}

double SpaceObject::height()
{
  return(sprite->height());
}

void SpaceObject::animate(Frame *space)
{
  sprite->animate((int)(corner_x()), (int)(corner_y()), sprite_index, space_zoom, space);
	return;
}
void SpaceObject::animate_radar (Frame *radar) {
	if(radar_color >= 0) {
		putpixel(radar->frame,
				(int)(normal_x() / (X_MAX / 56.0)),
				(int)(normal_y() / (Y_MAX / 48.0)), pallete_color[radar_color]);
		radar->add_pixel(
				(int)(normal_x() / (X_MAX / 56.0)),
				(int)(normal_y() / (Y_MAX / 48.0)), 1, 1);
		}
	return;
	}

void SpaceObject::collide(SpaceObject *other) {
	double v1, v2;
	double dx, dy;
	double dvx, dvy;
	double tmp;

	int x1, y1;
	int x2, y2;

	if((!canCollide(other)) || (!other->canCollide(this))) return;

	x1 = (int)(normal_x() - (w / 2.0));
	y1 = (int)(normal_y() - (h / 2.0));
	dx = min_delta(normal_x(), other->normal_x(), X_MAX);
	dy = min_delta(normal_y(), other->normal_y(), Y_MAX);
	x2 = (int)(normal_x() - dx - ((other->w) / 2.0));
	y2 = (int)(normal_y() - dy - ((other->h) / 2.0));

	if (!sprite->collide(x1, y1, sprite_index, x2, y2, 
			other->sprite_index, other->sprite)) 
		return;

	inflict_damage(other);
	other->inflict_damage(this);

	if ((mass <= 0) || (other->mass <= 0)) return;

	v1 = sqrt(((vx) * (vx)) +
		       ((vy) * (vy)));
  v2 = sqrt(((other->vx) * (other->vx)) +
		       ((other->vy) * (other->vy)));
	dvx = vx - other->vx;
	dvy = vy - other->vy;

	while ((dx == 0) && (dy == 0)) {
		dx = ((random() % 201) - 100) / 4.0;
		dy = ((random() % 201) - 100) / 4.0;
		}

	x -= vx * frame_time;
	y -= vy * frame_time;
	other->x -= other->vx * frame_time;
	other->y -= other->vy * frame_time;
	x1 = (int)(normal_x() - (w / 2.0));
	y1 = (int)(normal_y() - (h / 2.0));
	dx = min_delta(normal_x(), other->normal_x(), X_MAX);
	dy = min_delta(normal_y(), other->normal_y(), Y_MAX);
	x2 = (int)(normal_x() - dx - ((other->w) / 2.0));
	y2 = (int)(normal_y() - dy - ((other->h) / 2.0));

	double ndx, ndy;
	ndx = dx / (mass + other->mass) / sqrt(dx * dx + dy * dy);
	ndy = dy / (mass + other->mass) / sqrt(dx * dx + dy * dy);
	while (sprite->collide(x1, y1, sprite_index, x2, y2, 
			other->sprite_index, other->sprite)) {
		x += ndx * other->mass;
		y += ndy * other->mass;
		other->x -= ndx * mass;
		other->y -= ndy * mass;
		x1 = (int)(normal_x() - (w / 2.0));
		y1 = (int)(normal_y() - (h / 2.0));
		dx = min_delta(normal_x(), other->normal_x(), X_MAX);
		dy = min_delta(normal_y(), other->normal_y(), Y_MAX);
		x2 = (int)(normal_x() - dx - ((other->w) / 2.0));
		y2 = (int)(normal_y() - dy - ((other->h) / 2.0));
		}
	tmp = dvx * dx + dvy * dy;
	tmp = fabs( 2 * tmp / (dx * dx + dy * dy));
	tmp = tmp * (mass * other->mass) / (mass + other->mass);
	vx += dx * tmp / mass;
	vy += dy * tmp / mass;
	other->vx -= dx * tmp / other->mass;
	other->vy -= dy * tmp / other->mass;
	return;
	}

double SpaceObject::collide_line(double lx1, double ly1, double lx2,
  double ly2, double llength)
{
  int collide_x = (int)(lx2);
  int collide_y = (int)(ly2);
  double dx, dy;

  if (sprite->collide_line(
			(int)(lx1), (int)(ly1), &collide_x, &collide_y,
			(int)(lx1 - min_delta(lx1, normal_x(), X_MAX)),
			(int)(ly1 - min_delta(ly1, normal_y(), Y_MAX)), 
			sprite_index)) {
		dx = lx2 - (double)(collide_x);
    dy = ly2 - (double)(collide_y);
    llength = llength - sqrt((dx * dx) + (dy * dy));
  }

  return(llength);
}

void SpaceObject::inflict_damage(SpaceObject *other) {
	int i;

	if(damage_factor > 0) {
		i = damage_factor / 2;
		if(i >= BOOM_SAMPLES) i = BOOM_SAMPLES - 1;
		tw_sample((SAMPLE *)(melee[MELEE_BOOM + i].dat));
		other->damage += damage_factor;
		}
	other->handle_damage(this);
	return;
	}

void SpaceObject::handle_damage(SpaceLocation *source) {
	return;
	}

void SpaceObject::calculate() {
	x += vx * frame_time;
	x = normalize(x, X_MAX);
	y += vy * frame_time;
	y = normalize(y, Y_MAX);
	return;
	}

SpaceLine::SpaceLine(double lx, double ly, double langle, int team,
  double llength, int lcolor) :
  SpaceLocation(lx, ly, langle, team),
  length(llength),
  color(lcolor),
  collide_object(NULL)
{
	id = SPACE_LINE;
	layer = LAYER_LINES;
	collide_flag_anyone &= OBJECT_LAYERS;
	collide_flag_ally   &= OBJECT_LAYERS;
}

 double SpaceLine::edge_x()
{
  return(cos(angle * ANGLE_RATIO) * length);
}

 double SpaceLine::edge_y()
{
  return(sin(angle * ANGLE_RATIO) * length);
}

 double SpaceLine::get_length()
{
  return(length);
}

void SpaceLine::calculate()
{
  int i;

  if (collide_object) {
		i = damage_factor / 2;
    if(i >= BOOM_SAMPLES)
      i = BOOM_SAMPLES - 1;
		tw_sample((SAMPLE *)(melee[MELEE_BOOM + i].dat));
		collide_object->damage += damage_factor;
    collide_object->handle_damage(this);
		collide_flag_anyone = collide_flag_ally = 0;
    spaceFrame->addSpaceObject(new Animation(
      normal_x() + edge_x(), normal_y() + edge_y(), sparkSprite,
      SPARK_FRAMES, scale_frames(0), LAYER_EXPLOSIONS));
  }

  if(collide_object)
    collide_object = NULL;
}

void SpaceLine::animate(Frame *space)
{
  double x1, y1, x2, y2;

  x1 = normal_x() - space_x + 800;
	while (x1 < 0) x1 += X_MAX;
	x1 -= 800;
  x2 = x1 + edge_x();
  x1 *= space_zoom;
  x2 *= space_zoom;

  y1 = normal_y() - space_y + 800;
	while (y1 < 0) y1 += Y_MAX;
	y1 -= 800;
  y2 = y1 + edge_y();
  y1 *= space_zoom;
  y2 *= space_zoom;

  line(space->frame, (int)(x1), (int)(y1), (int)(x2), (int)(y2), color);
  space->add_line((int)(x1), (int)(y1), (int)(x2), (int)(y2));
}

void SpaceLine::collide(SpaceObject *o)
{
  double old_length = length;
  
  if((!canCollide(o)) || (!o->canCollide(this)))
    return;

  length = o->collide_line(normal_x(), normal_y(), normal_x() + edge_x(),
    normal_y() + edge_y(), length);
  if(length != old_length)
    collide_object = o;
	return;
}

SpaceFrame::SpaceFrame(Frame *spaceFrame, Frame *radarFrame, int layerCount) :
		firstLine(NULL),
		space(spaceFrame),
		radar(radarFrame),
		numLayers(layerCount) {
	int i;
	quadranto = new SpaceObject*[QUADS_TOTAL];
	for (i = 0; i < QUADS_TOTAL; i += 1) {
		quadranto[i] = NULL;
		}

	firstLayerObject = new SpaceObject*[numLayers];
	for(i = 0; i < numLayers; i++) firstLayerObject[i] = NULL;
	}

SpaceFrame::~SpaceFrame() {
	SpaceObject *ocurrent;
	SpaceObject *otemp;
	SpaceLine *lcurrent; 
	SpaceLine *ltemp;
	int i;

	lcurrent = firstLine;
	while(lcurrent != NULL) {
		ltemp = lcurrent;
		lcurrent = lcurrent->nextl;
		delete ltemp;
		ltemp = NULL;
		}

	for(i = 0; i < numLayers; i++) {
		ocurrent = firstLayerObject[i];
		while(ocurrent != NULL) {
			otemp = ocurrent;
			ocurrent = ocurrent->nexto;
			delete otemp;
			otemp = NULL;
			}
		}
//	delete firstLayerObject;
//	delete quadranto;
	}

SpaceObject *SpaceFrame::addSpaceObject(SpaceObject *o) {
	ASSERT((o->layer < SPACE_LAYERS) && (o->layer >= 0));
	o->nexto = firstLayerObject[o->layer];
	firstLayerObject[o->layer] = o;
	return(o);
	}

bool SpaceFrame::removeSpaceObject(SpaceObject *o) {
	SpaceObject *prev = NULL, *tmp = firstLayerObject[o->layer];
	while (tmp) {
		if (tmp == o) {
			if (prev) prev->next = o->next;
			o->next = NULL;
			return true;
			}
		prev = tmp;
		tmp = tmp->nexto;
		}
	return false;
	}

SpaceLine *SpaceFrame::addSpaceLine(SpaceLine *l) {
	l->nextl = firstLine;
	firstLine = l;
	return(l);
	}

SpaceObject *SpaceFrame::getFirstObject(int layer)
{
  return firstLayerObject[layer];
}

SpaceLine *SpaceFrame::getFirstLine()
{
  return firstLine;
}

void SpaceFrame::calculate()
{
  SpaceObject *ocurrent;
  SpaceLine *lcurrent, *lprev;
  int i;

	for(i = 0; i < QUADS_TOTAL; i += 1) {
		quadranto[i] = NULL;
		}
	for(i = 0; i < numLayers; i += 1) {
		ocurrent = firstLayerObject[i];
		while(ocurrent) {
			if(ocurrent->exists()) {
				int q = int(ocurrent->normal_x() * QUADI_X) + 
						int(ocurrent->normal_y() * QUADI_Y) * QUADS_X;
				if (quadranto[q]) {
					ocurrent->qnext = quadranto[q];
					}
				quadranto[q] = ocurrent;
				ocurrent->qnext = NULL;
				}
			ocurrent = ocurrent->nexto;
			}
		}

	for(i = 0; i < numLayers; i++) {
		ocurrent = firstLayerObject[i];
		while(ocurrent) {
			if(ocurrent->exists())
				ocurrent->calculate();
			ocurrent = ocurrent->nexto;
			}
		}


  lcurrent = firstLine;
  lprev = NULL;

	while(lcurrent) {
		if(lcurrent->exists()) lcurrent->calculate();
		lcurrent = lcurrent->nextl;
		}

	SpaceLocation *current, *temp, *prev;

	for(i = 0; i < numLayers; i++) {
		if (i != LAYER_LINES) 
			current = firstLayerObject[i];
		else current = firstLine;

		prev = NULL;
		while(current) {
			if(current->state == -DEATH_FRAMES) {
				temp = current;
				current = current->next;
				if(prev) prev->next = current;
				if(firstLayerObject[i] == temp) firstLayerObject[i] = (SpaceObject *)current;
				if(firstLine == temp) firstLine = (SpaceLine*) current;
				delete temp;
				temp = NULL;
				}
			else {
				if (!current->exists()) {
					if (current->state == 0) current->death();
					current->state -= 1;
					}
				prev = current;
				current = current->next;
				}
			}
		}
	return;
	}

void SpaceFrame::animate()
{
  SpaceObject *ocurrent;
  SpaceLine *lcurrent;
  int i;

  for(i = 0; i < numLayers; i++) {
    ocurrent = firstLayerObject[i];
    while(ocurrent != NULL) {
      if(ocurrent->exists()) {
				ocurrent->animate(space);
				ocurrent->animate_radar(radar);
				}
      ocurrent = ocurrent->nexto;
    }
		if (i == LAYER_LINES) {
			lcurrent = firstLine;
			while(lcurrent != NULL) {
				if(lcurrent->exists())
					lcurrent->animate(space);
				lcurrent = lcurrent->nextl;
			}
		}
  }

}

void SpaceFrame::collide()
{
  SpaceObject *ocurrent, *oother;
  SpaceLine *lcurrent;//, *lother;
  int i, j;

  for(i = 0; i < numLayers; i++) {
    ocurrent = firstLayerObject[i];
    while(ocurrent != NULL) {
      if(ocurrent->exists()) {
        oother = ocurrent->nexto;
        for(j = i; j < numLayers; j++) {
          if(i < j)
            oother = firstLayerObject[j];
          while(oother != NULL) {
            if(oother->exists())
              ocurrent->collide(oother);
            oother = oother->nexto;
          }
        }
      }
      ocurrent = ocurrent->nexto;
    }
  }

  for(i = 0; i < numLayers; i++) {
    lcurrent = firstLine;
    while(lcurrent != NULL) {
      if(lcurrent->exists()) {
        oother = firstLayerObject[i];
        while(oother != NULL) {
          if(oother->exists())
            lcurrent->collide(oother);
          oother = oother->nexto;
        }
      }
      lcurrent = lcurrent->nextl;
    }
  }
}
