#ifndef _MFRAME_H
#define _MFRAME_H

#define Query Query_old

class Query_old {
	int layers;
	int layer;
	double range;
	SpaceLocation *target;
	void next_layer ();
	public:
	union {
		SpaceObject   *currento;
		SpaceLine     *currentl;
		SpaceLocation *current;
		};
	void begin (SpaceLocation *target, int layers, double range);
	void next ();
	void end();
	};

class Query_new {
	int layers;
	double range;
	SpaceLocation *target;
	int qx_min, qx_max;
	int qy_min, qy_max;
	int qx, qy;
	void next_quadrant ();
	public:
	union {
		SpaceObject   *currento;
		SpaceLine     *currentl;
		SpaceLocation *current;
		};
	void begin (SpaceLocation *target, int layers, double range);
	void next ();
	void end();
	};

class SpaceLocation {
  protected:
  int    id;
  double x;
  double y;
  double angle;
  double vx;
  double vy;
  public:
  int    state;
	int    layer;
  public:
  int    damage_factor;
  int    collide_flag_anyone;
  int    collide_flag_ally;
  int    ally_flag;
  int    damage;
  int    directDamage;
  int    repair;
  int    fuel_sap;
  double speed_loss;

  SpaceLocation(double lx, double ly, double langle, int team);
  virtual void death();
	
  SpaceLocation *newLocation(double lx, double ly, double langle);

  double get_angle();
  double get_vx();
  double get_vy();

  int getID();
  int allyFlag();

  double normal_x();
  double normal_y();
  double nearest_x(SpaceLocation *l);
  double nearest_y(SpaceLocation *l);
  double distance(SpaceLocation *l);
  double trajectory_angle(SpaceLocation *l);

	int isLine();
	int isObject();
	int isAsteroid();
	int isPlanet();
	int isShip();
	int isShot();

	virtual int exists();
	virtual int canCollide(SpaceLocation *other);
	virtual int isAlly(SpaceLocation *other);
	union {
		SpaceObject   *nexto;
		SpaceLine     *nextl;
		SpaceLocation *next;
		};
	union {
		SpaceObject   *qnexto;
		SpaceLine     *qnextl;
		SpaceLocation *qnext;
		};

	virtual int translate(double dx, double dy);
	virtual int accelerate(SpaceLocation *source, double angle, double vel, double max_speed);

	virtual void animate(Frame *space);
	virtual void calculate();
	};

class SpaceObject : public SpaceLocation {
	public:
	double       w;
	double       h;
	double       mass;
	protected:
	SpaceSprite *sprite;
	int          sprite_index;
	int          radar_color;

	public:
	SpaceObject(double ox, double oy, double oangle, int ocflag,
			SpaceSprite *osprite);

	double corner_x();
	double corner_y();

	SpaceSprite *getSprite();

	int getSpriteIndex();

	double width();
	double height();

	virtual void animate(Frame *space);
	virtual void animate_radar(Frame *radar);
	virtual void collide(SpaceObject *other);
	virtual double collide_line(double lx1, double ly1, double lx2, double ly2,
			double llength);

	virtual void inflict_damage(SpaceObject *other);
	virtual void handle_damage(SpaceLocation *source);

	virtual void calculate();
	};

class SpaceLine : public SpaceLocation {
	protected:
	double length;
	int    color;

	SpaceObject *collide_object;

	public:
	SpaceLine(double lx, double ly, double langle, int lcflag, double llength,
			int lcolor);

	double edge_x();
	double edge_y();
	double get_length();

	virtual void calculate();
	virtual void animate(Frame *space);
	virtual void collide(SpaceObject *o);
	};

class SpaceFrame {
  private:
	SpaceObject **quadranto;
	friend Query_old;
	friend Query_new;
  SpaceObject **firstLayerObject;
  SpaceLine    *firstLine;
  Frame        *space;
  Frame        *radar;
  int           numLayers;

  public:
  SpaceFrame(Frame *spaceFrame, Frame *radarFrame, int layerCount);
  ~SpaceFrame();

  SpaceObject *addSpaceObject(SpaceObject *o);
  bool removeSpaceObject(SpaceObject *o);
  SpaceObject *getFirstObject(int layer);

  SpaceLine *addSpaceLine(SpaceLine *l);
  SpaceLine *getFirstLine();

  void calculate();
  void collide();
  void animate();
};

#endif