#include "../melee.h"
#include "../scp.h"
#include "mmain.h"
#include "mnet.h"
#include "mview.h"
#include "mlog.h"
#include <string.h>

static int log_num = 0;   //highest channel number in use
static int log_size[256]; //size of memory allocated for each channel (bytes)
static int log_len[256];  //length of channel (bytes)
static int log_pos[256];  //position of reading from channel (bytes)
static int log_pos2[256]; //position of reading from channel for network purposes (bytes)
static int log_dir[256];  //direction of channel (none, self->other, other->self, self->self)
static int log_dir_files;  //direction of channel (none, self->other, other->self, self->self)
#define LOG_DIR_SEND  1
#define LOG_DIR_RECV  2
#define LOG_DIR_READ  4
#define LOG_DIR_WRITE 8
#define LOG_DIR_SAVE  16
static unsigned char *log_data[256];
static int game_latency;
static NetTCP net;

void _log(int id, void *data, int size) ;

// log start
// log scp.ini (file)
// log random number seed (main)
// 

static int net1_paused = 0;

void net1_handle_code(unsigned int code) {
	switch (code) {
		case NET1_CODE_QUIT: {
			message.out("Quitting");
			throw "Quit: recieved quit signal over network";
			}
		break;
		case NET1_CODE_PAUSE: {
			net1_paused += 1;
			BITMAP *bmp = create_bitmap(320, 50);
			blit (screen, bmp, (screen_x-bmp->w)/2, (screen_y-bmp->h)/2, 0, 0, bmp->w, bmp->h);
			rect(screen, (screen_x-bmp->w)/2, (screen_y-bmp->h)/2, (screen_x+bmp->w)/2, (screen_y+bmp->h)/2, palette_color[8]);
			textprintf(screen, font, (screen_x-bmp->w)/2, (screen_y-bmp->h)/2, palette_color[9], "Other side paused");
			while (net1_paused) {
				while(!net.ready2recv()) ;
				net1_recv_packet();
				}
			blit (bmp, screen, 0, 0, (screen_x-bmp->w)/2, (screen_y-bmp->h)/2, bmp->w, bmp->h);
			}
		break;
		case NET1_CODE_UNPAUSE: {
			if (!net1_paused) throw "net1_unpause -- game not paused!";
			net1_paused -= 1;
			}
		break;
		}
	return;
	}

void net1_send_code(unsigned int code) {
	if (!(code & 0x80000000)) throw "net1_send_special -- bade code!";
	net.send(sizeof(unsigned int), &code);
	return;
	}

static int read_length_code (int max, int *clen, int *len, unsigned char *where) {
	if (max < 1) return -1;
	*clen = 1;
	*len = where[0];
	if (*len < 255) return 0;
	*clen = 3;
	if (*clen > max) return -1;
	*len = where[1] + (where[2] << 8);
	if (*len < 65535) return 0;
	*clen = 7;
	if (*clen > max) return -1;
	*len = where[3] + (where[4] << 8) + (where[5] << 16) + (where[6] << 24);
	return 0;
	}
static int write_length_code (int max, int *clen, int len, unsigned char *where) {
	if (len <= 0) throw "write_length_code -- bad length";
	if (max < 1) return -1;
	if (len < 255) {
		where[0] = len;
		*clen = 1;
		return 0;
		}
	else {
		if (max < 3) return -1;
		where[0] = 255; 
		if (len < 65535) {
			where[1] = len & 255;
			where[2] = len >> 8;
			*clen = 3;
			return 0;
			}
		else {
			if (max < 7) return -1;
			where[1] = 255;
			where[2] = 255;
			where[3] = (len >> 0) & 255;
			where[4] = (len >> 8) & 255;
			where[5] = (len >> 16) & 255;
			where[6] = (len >> 24) & 255;
			*clen = 7;
			return 0;
			}
		}
	}

void net1_send_packet() {
	unsigned char buf[1280];
	int i, j, k, pos = 4;
	for (i = 0; i < 255; i += 1) if (log_dir[i] & LOG_DIR_SEND) {
		j = log_len[i] - log_pos2[i];
		if (j > 0) {
			buf[pos] = i;
			pos += 1;
			write_length_code(12, &k, j, &buf[pos]);
			pos += k;
			memcpy(buf+pos, log_data[i] + log_pos2[i], j);
			log_pos2[i] += j;
			pos += j;
			}
		}
	if (pos <= 0) throw "net1_send_packet -- no data to send";
	buf[0] = (pos >> 0) & 255;
	buf[1] = (pos >> 8) & 255;
	buf[2] = (pos >> 16) & 255;
	buf[3] = (pos >> 24) & 255;
	net.send(pos, &buf);
	return;
	}

void net1_recv_packet() {
	unsigned char buf[2048];
	int pos, len;
	int i, j, k, l;
	len = net.recv(4, 4, &buf);
	if (len != 4) throw "net1_recv_packet -- net.recv error (1)";
	len = buf[0] + (buf[1] << 8) + (buf[2] << 16) + (buf[3] << 24);
	if (len & 0x80000000) {
		net1_handle_code(len);
		return;
		}
	len -= 4;
	if (len != net.recv(len, len, &buf)) throw "net1_recv_packet -- net.recv error (2)";
	pos = 0;
	while (pos < len) {
		i = buf[pos];
		if (!(log_dir[i] & LOG_DIR_RECV)) throw ("net1_recv_packet -- data on wrong channel");
		pos += 1;
		l = read_length_code(len-pos, &j, &k, &buf[pos]);
		if (l < 0) throw ("net1_recv_packet -- read_length_code failed");
		pos += j;
		_log(i, &buf[pos], k);
		pos += k;
		}
	if (pos != len) throw ("net1_recv_packet -- missaligned packet!!!");
	return;
	}

void normal_init(char *file, int source) {
	char tmp[128], tmp2[128];
	char *t;
	int i, c;
	int numplayers;
	i = 0;
	numplayers = 0;
	while (true) {
		i += 1;
		if (source) {
			tw_unlog_file(file);
			}
		else set_config_file(file);
		sprintf (tmp, "Player%d", i);
		sprintf (tmp2, "Player%d", numplayers+1);
		t = (char*)get_config_string (tmp, "Type", NULL);
		c = get_config_int (tmp, "Config", 0);
		if (!t) break;
		if (!strcmp(t, "None")) continue;
		t = strdup(t);
		set_config_file("tmp.ini");
		set_config_string(tmp2, "Type", t);
		free(t);
		set_config_string(tmp2, "Name", tmp);
		set_config_int(tmp2, "config", c);
		if (source) {
			tw_log_file("fleets.ini");
			tw_unlog_file("fleets.ini");
			}
		else set_config_file("fleets.ini");
		load_fleet(NULL, tmp);
		save_fleet("tmp.ini", tmp2);
		numplayers += 1;
		}
	set_config_file("tmp.ini");
	set_config_int("Game", "NumPlayers", numplayers);
	set_config_int("Game", "FirstLocalPlayer", 0);
	set_config_int("Game", "NumLocalPlayers", numplayers);
	set_config_string("Game", "Version", (char*) tw_version());
	set_config_file("tmp.ini");
	return;
	}

static void latency_increment() {
	char m = 0;
	int i;
	for (i = 0; i < num_players; i += 1) {
		tw_log(LOG_PLAYER_KEYS+i, &m, 1);
//		log_pos2[i] += 1;
		}
	tw_log(LOG_CHECKSUM1, &m, 1);
	tw_log(LOG_CHECKSUM2, &m, 1);
	game_latency += 1;
	return;
	}

int tw_log_start() {
	int i, j;
	char path[512] = "demo.log";
	char buf[32];
	delete_file("tmp.ini");
	delete_file("fleet.tmp");
	for (i = 0; i < 256; i += 1) {
		log_len[i] = 0;
		log_pos[i] = 0;
		log_pos2[i] = 0;
		log_size[i] = 0;
		log_data[i] = NULL;
		log_dir[i] = 0;
		}
	log_num = 0;
	log_dir_files = 0;
	game_latency = 0;
	i = 0;
	switch (play_mode) {
		case MELEE_NORMAL: {
			tw_log_file("scp.ini");
			tw_log_file("fleets.ini");
			normal_init("scp.ini", 1);
			}
		break;
		case MELEE_PLAYBACK: {
			int i = 1;
			while (i > 0) {
				if(file_select("Load Recording From", path, "LOG")) {
					if (!tw_log_load(path)) {
						i = 0; 
						break;
						}
					}
				if (alert ("Failed to load demo", NULL, NULL, "&Abort", "&Retry", 'a', 'r') == 1) i = -1;
				}
			if (i) break;
			log_dir_files = LOG_DIR_READ;
			for (i = 0; i < 256; i += 1) log_dir[i] = LOG_DIR_READ;
			log_dir[LOG_CHECKSUM1] = LOG_DIR_WRITE + LOG_DIR_READ;
			tw_log_file("scp.ini");
			tw_log_file("fleets.ini");
			normal_init("scp.ini", 1);
			set_config_file("tmp.ini");
			j = get_config_int("game", "NumPlayers", 0);
			set_config_int ("game", "FirstLocalPlayer", 0);
			set_config_int ("game", "NumLocalPlayers", j);
			for (i = 0; i < j; i += 1) {
				sprintf (buf, "Player%d", i+1);
				set_config_string (buf, "Type", "VegetableBot");
				}
			set_config_file("tmp.ini");
			i = 0;
			}
		break;
		case MELEE_RECORD: {
			log_dir_files = LOG_DIR_READ + LOG_DIR_WRITE + LOG_DIR_SAVE;
			for (i = 0; i < 256; i += 1) log_dir[i] = LOG_DIR_READ + LOG_DIR_WRITE + LOG_DIR_SAVE;
			log_dir[LOG_CHECKSUM1] = LOG_DIR_READ + LOG_DIR_WRITE;
			tw_log_file("scp.ini");
			tw_log_file("fleets.ini");
			normal_init("scp.ini", 1);
			}
		break;
		case MELEE_NET1_SERVER: {
			net.init();
			message.out("");message.out("");message.out("");
			message.out("Listening... (press the ESCAPE key to cancel)");
			i = net.listen(15515);
			if (i >= 0) {
				message.out ("NET1_SERVER Game Started");
				}
			else {
				alert("Failed to start NET1_SERVER game", NULL, NULL, "Abort", NULL, 'a', 0);
				return -1;
				}
			net.optimize4latency();
			log_dir_files = LOG_DIR_READ + LOG_DIR_WRITE + LOG_DIR_SEND;
			for (i = 0; i < 256; i += 1) {
				log_dir[i] = LOG_DIR_READ + LOG_DIR_WRITE + LOG_DIR_SEND;
				}
			log_dir[LOG_CHECKSUM1] = LOG_DIR_READ + LOG_DIR_RECV;
			log_dir[LOG_CHECKSUM2] = LOG_DIR_READ + LOG_DIR_WRITE + LOG_DIR_SEND;
			num_players = 2;
			first_local_player = 0;
			num_local_players = 1;
			set_config_file("tmp.ini");
			set_config_string("Player1", "Type", "Keyboard");
			set_config_int   ("Player1", "Config", 0);
			set_config_string("Player1", "name", "ServerPlayer!");
			set_config_string("Player2", "Type", "VegetableBot");
			set_config_int   ("Player2", "Config", 0);
			set_config_string("Player2", "name", "ClientPlayer");
			set_config_int ("Game", "NumPlayers", num_players);
			set_config_int ("Game", "FirstLocalPlayer", first_local_player);
			set_config_int ("Game", "NumLocalPlayers", num_local_players);
			set_config_file("tmp.ini");
			load_fleet("fleets/safe.scf", "Fleet");
			save_fleet("tmp.ini", "Player1");
			load_fleet("fleets/safe.scf", "Fleet");
			save_fleet("tmp.ini", "Player2");
			for (i = 0; i < 64; i += 1) {
				log_dir[LOG_PLAYER_SHIPS+i] = LOG_DIR_RECV + LOG_DIR_READ;
				log_dir[LOG_PLAYER_KEYS+i]  = LOG_DIR_RECV + LOG_DIR_READ;
				}
			for (i = first_local_player; i < first_local_player+num_local_players; i+=1) {
				log_dir[LOG_PLAYER_SHIPS+i] = LOG_DIR_WRITE + LOG_DIR_SEND;
				log_dir[LOG_PLAYER_KEYS+i]  = LOG_DIR_WRITE + LOG_DIR_SEND + LOG_DIR_READ;
				}
			set_config_file ("scp.ini");
			j = get_config_int("Network", "Lag", 5);
			tw_log(LOG_MAIN, &j, 4);
			tw_unlog(LOG_MAIN, &j, 4);
			for (i = 0; i < j; i += 1) {
				latency_increment();
				}
			}
		break;
		case MELEE_NET1_CLIENT: {
			net.init();
			message.out("");message.out("");message.out("");
			message.out("Connecting... (hold down the ESCAPE key to abort)");
			while ((j = net.connect(NULL, 15515)) < 0) {
				if (!j) break;
				if (j == -1) return -1;
				}
			net.optimize4latency();
			message.out ("Network Client Game Started");
			log_dir_files = LOG_DIR_READ + LOG_DIR_RECV;
			for (i = 0; i < 256; i += 1) {
				log_dir[i] = LOG_DIR_READ + LOG_DIR_RECV;
				}
			log_dir[LOG_CHECKSUM2] = LOG_DIR_READ + LOG_DIR_RECV;
			log_dir[LOG_CHECKSUM1] = LOG_DIR_READ + LOG_DIR_WRITE + LOG_DIR_SEND;
			num_players = 2;
			first_local_player = 1;
			num_local_players  = 1;
			set_config_file("tmp.ini");
			set_config_string("Player1", "Type", "VegetableBot");
			set_config_int   ("Player1", "Config", 0);
			set_config_string("Player1", "name", "ServerPlayer");
			set_config_string("Player2", "Type", "Keyboard");
			set_config_int   ("Player2", "Config", 0);
			set_config_string("Player2", "name", "ClientPlayer!");
			set_config_int ("Game", "NumPlayers", num_players);
			set_config_int ("Game", "FirstLocalPlayer", first_local_player);
			set_config_int ("Game", "NumLocalPlayers", num_local_players);
			set_config_file("tmp.ini");
			load_fleet("fleets/safe.scf", "Fleet");
			save_fleet("tmp.ini", "Player1");
			load_fleet("fleets/safe.scf", "Fleet");
			save_fleet("tmp.ini", "Player2");
			for (i = 0; i < 64; i += 1) {
				log_dir[LOG_PLAYER_SHIPS+i] = LOG_DIR_RECV + LOG_DIR_READ;
				log_dir[LOG_PLAYER_KEYS+i]  = LOG_DIR_RECV + LOG_DIR_READ;
				}
			for (i = first_local_player; i < first_local_player+num_local_players; i+=1) {
				log_dir[LOG_PLAYER_SHIPS+i] = LOG_DIR_WRITE + LOG_DIR_SEND;
				log_dir[LOG_PLAYER_KEYS+i]  = LOG_DIR_WRITE + LOG_DIR_SEND + LOG_DIR_READ;
				}
			set_config_file ("scp.ini");
			j = get_config_int("Network", "Lag", 5);
			tw_log(LOG_MAIN, &j, 4);
			tw_unlog(LOG_MAIN, &j, 4);
			for (i = 0; i < j; i += 1) {
				latency_increment();
				}
			}
		break;
		default: alert ("This mode of play is not yet supported", NULL, NULL, "Abort", NULL, 'a', 0); i = -1;
		}
	return i;
	}

void tw_log_end() {
	int i;
	switch (play_mode) {
		default: {
			}
		break;
		case MELEE_RECORD: {
			char path[256] = "demo.log";
			while (file_select("Save Recording As", path, "LOG")) {
				if (!tw_log_save(path)) {
					clear_to_color(screen, pallete_color[5]);
					break;
					}
				else if (alert ("Failed to save demo", NULL, NULL, "&Abort", "&Retry", 'a', 'r') == 1) break;
				}
			}
		break;
		case MELEE_NET1_SERVER:
		case MELEE_NET1_CLIENT: {
			net1_send_code(NET1_CODE_QUIT);
			rest(250);
			net.disconnect();
			}
		break;
		}
	for (i = 0; i < 256; i += 1) {
		free(log_data[i]);
		log_data[i] = NULL;
		log_len[i] = 0;
		log_size[i] = 0;
		}
	return;
	}

int tw_log_save (char *fname) {
	PACKFILE *f;
	int i;
	f = pack_fopen ( fname, F_WRITE_PACKED);
	if (!f) return -1;
	int lognum = 0;
	for (i = 0; i < 256; i += 1) if ((log_dir[i] & LOG_DIR_SAVE) && log_len[i]) lognum += 1;
	pack_iputl ( lognum, f);
	for (i = 0; i < 256; i += 1) {
		if (!(log_dir[i] & LOG_DIR_SAVE)) continue;
		if (!log_len[i]) continue;
		f = pack_fopen_chunk(f, 0);
		pack_iputl ( i, f);
		pack_iputl ( log_len[i], f);
		pack_fwrite ( log_data[i], log_len[i], f);
		f = pack_fclose_chunk(f);
		}
	pack_fclose (f);
	return 0;
	}

int tw_log_load (char *fname) {
	PACKFILE *f;
	int i;
	f = pack_fopen ( fname, F_READ_PACKED);
	if (log_num) throw ("tw_log_load - logs already exist");
	if (!f) return -1;
	log_num = pack_igetl ( f);
	for (int j = 0; j < log_num; j += 1) {
		f = pack_fopen_chunk(f, 0);
		i = pack_igetl ( f);
		log_len[i] = pack_igetl ( f);
		log_size[i] = log_len[i];
		log_data[i] = (unsigned char *) malloc(log_size[i]);
		pack_fread ( log_data[i], log_len[i], f);
		f = pack_fclose_chunk(f);
		}
	pack_fclose (f);
	return 0;
	}

int tw_log_ready(int id) {
	return log_len[id] - log_pos[id];
	}

void _log(int id, void *data, int size) {
	log_len[id] += size;
	while (log_len[id] > log_size[id]) {
		if (log_size[id] == 0) log_num += 1;
		if (log_size[id]) log_size[id] = log_size[id] * 2;
		else log_size[id] = 16;
		log_data[id] = (unsigned char *) realloc(log_data[id], log_size[id]);
		}
	memmove(log_data[id]+(log_len[id]-size), data, size);
	return;
	}

void tw_log(int id, void *data, int size) {
	if (!(log_dir[id] & LOG_DIR_WRITE)) return;
	_log(id, data, size);
	if (log_dir[id] & LOG_DIR_SEND) {
		if (id < LOG_PLAYER_KEYS) net1_send_packet();
		}
	return;
	}

void _unlog(int id, void *data, int size) {
	if (tw_log_ready(id) < size) {
		throw "Quit: _unlog on non-existant data";
		return;
		}
	memmove(data, log_data[id] + log_pos[id], size);
	log_pos[id] += size;
	return;
	}

void tw_unlog(int id, void *data, int size) {
	if (log_dir[id] & LOG_DIR_RECV) {
		while (log_len[id] - log_pos[id] < size) {
			while (!net.ready2recv()) {
				rest(5);
				}
			net1_recv_packet();
			}
		}
	if (!(log_dir[id] & LOG_DIR_READ)) return;
	_unlog(id, data, size);
	return;
	}

int tw_log_file_ready(char *name, int *location) {
	int i = 0, j = 0;
	while (i < log_len[LOG_FILE_NAMES]) {
		if (strcmp((char*) log_data[LOG_FILE_NAMES] + i, name)) {
			i += strlen((char*) log_data[LOG_FILE_NAMES] + i) + 1;
			j += *((int*)(log_data[LOG_FILE_NAMES] + i));
			i += 4;
			}
		else {
			i += strlen((char*) log_data[LOG_FILE_NAMES] + i) + 1;
			int k = *((int*)(log_data[LOG_FILE_NAMES] + i));
			if (j+k > log_len[LOG_FILE_DATA]) return -2;
			if (location) *location = j;
			return k;
			}
		}
	if (location) *location = -1;
	return -1;
	}

void tw_log_file(char *name){
	char buffy[1024+64];
	PACKFILE *f;
	int i, j = 0;
	if (log_dir_files & LOG_DIR_RECV) {
		while (tw_log_file_ready(name) < 0) {
			if (net.ready2recv()) {
				net1_recv_packet();
				}
			rest(5);
			}
//		textprintf (screen, font, 0, 180, palette_color[15], "file ready: %s", name);
		return;
		}
	if (!(log_dir_files & LOG_DIR_WRITE)) return;
	if (tw_log_file_ready(name) >= 0) return;
	f = pack_fopen(name, F_READ);
	if (!f) {
		sprintf(buffy, "tw_log_file - bad file name %s", name);
		throw buffy;
		}
	do {
		i = pack_fread(buffy, 1024, f);
		if (i > 0) {
			tw_log(LOG_FILE_DATA, buffy, i);
			}
		j += i;
		} while (i > 0);
	strcpy(buffy, name);
//endian
	*((int*)(buffy + strlen(buffy) + 1)) = j;
	_log (LOG_FILE_NAMES, name, strlen(name)+1);
//endian (disk format)
	_log(LOG_FILE_NAMES, &j, 4);
	if (log_dir_files & LOG_DIR_SEND) net1_send_packet();
	return;
	}

void tw_unlog_file(char *name) {
	int i = 0, j = 0;
	if (!(log_dir_files & LOG_DIR_READ)) {
		if (exists(name)) {
			set_config_file(name);
			return;
			}
		}
	else {
		i = tw_log_file_ready(name, &j);
		if (i >= 0) set_config_data((char*) log_data[LOG_FILE_DATA] + j, i);
		return;
		}
	char *buf = (char*)malloc(512);
	sprintf(buf, "tw_unlog_file -- bad file name %s", name);
	throw(buf);
	free(buf);
	return;
	}


