
#include "../melee.h"
#include "../id.h"
#include "../frame.h"

#include "mcbodies.h"
#include "mmain.h"
#include "mcontrol.h"
#include "mship.h"
#include "mshot.h"
#include "manim.h"
#include "mview.h"

double gravity_sling = 0;

int hot_color[HOT_COLORS] =
{
  122, 123, 124, 125, 126, 127,
  42,  43,  44,  45,  46,  47
};

SpaceSprite *hotspotSprite;
SpaceSprite *kaboomSprite;

Ship::Ship(double ox, double oy, double shipAngle, int team,
  int shipRadarColor, ShipData *shipData, Control *cntrl) :
  SpaceObject(ox, oy, shipAngle, team, shipData->spriteShip),
	gravity_vx(0),
	gravity_vy(0),
  data(shipData),
  target(NULL),
  update_panel(FALSE), 
	target_pressed(FALSE), 
	control(cntrl)
{
  layer = LAYER_SHIPS;
	collide_flag_ally = bit(SPACE_LAYERS) - 1;
  int    i;
	nextkeys = -1;
	phased_in = false;

  id = SPACE_SHIP;

//  set_config_data((char *)(data->data[0].dat), data->data[0].size);
  crew     = get_config_int("Ship", "Crew", 0);
  crew_max = get_config_int("Ship", "CrewMax", 0);
  batt     = get_config_int("Ship", "Batt", 0);
  batt_max = get_config_int("Ship", "BattMax", 0);

  recharge_amount  = get_config_int("Ship", "RechargeAmount", 0);
  recharge_rate    = scale_frames(get_config_float("Ship", "RechargeRate", 0));
  recharge_step    = recharge_rate;
  weapon_drain     = get_config_int("Ship", "WeaponDrain", 0);
  weapon_rate      = scale_frames(get_config_float("Ship", "WeaponRate", 0));
  weapon_sample    = 0;
  weapon_recharge  = 0;
  weapon_low       = FALSE;
  special_drain    = get_config_int("Ship", "SpecialDrain", 0);
  special_rate     = scale_frames(get_config_float("Ship", "SpecialRate", 0));
  special_sample   = 0;
  special_recharge = 0;
  special_low      = FALSE;

  double raw_hotspot_rate = get_config_float("Ship", "HotspotRate", 0);
  hotspot_rate  = scale_frames(raw_hotspot_rate);
  hotspot_frame = 0;
  turn_rate     = scale_turning(get_config_float("Ship", "TurnRate", 0));
  turn_step     = 0.0;
  speed_max     = scale_velocity(get_config_float("Ship", "SpeedMax", 0));
  accel_rate    = scale_acceleration(get_config_float("Ship", "AccelRate", 0), raw_hotspot_rate);
  radar_color   = shipRadarColor;
  mass          = (get_config_float("Ship", "Mass", 0));

  thrust       = FALSE;
  turn_left    = FALSE;
  turn_right   = FALSE;
  fire_weapon  = FALSE;
  fire_special = FALSE;

  panelBitmap[0] = create_bitmap(64, 100);
  blit(data->bitmapPanel[0], panelBitmap[0], 0, 0, 0, 0, 64, 100);

  for(i = 1; i < 7; i++) {
    panelBitmap[i] = create_bitmap(CAPTAIN_WIDTH, CAPTAIN_HEIGHT);
    blit(data->bitmapPanel[i], panelBitmap[i], 0, 0, 0, 0,
      CAPTAIN_WIDTH, CAPTAIN_HEIGHT);
  }

  angle = shipAngle;
  sprite_index = (int)(angle / 5.625) + 16;
	sprite_index &= 63;

  phase_count = 0;
  phase_step  = 0;
}

void Ship::death()
{
  int i;

  for(i = 0; i < 7; i++)
    destroy_bitmap(panelBitmap[i]);

	return;
}

 int Ship::getCrew()
{
  return(crew);
}

 int Ship::getBatt()
{
  return(batt);
}

 int Ship::crewPanelColor()
{
  return pallete_color[2];
}

 int Ship::battPanelColor()
{
  return pallete_color[4];
}

void Ship::locate()
{
  double r;

  do {
    x = (double)(random() % (int)(X_MAX));
    y = (double)(random() % (int)(Y_MAX));
    x = normal_x();
    y = normal_y();
    r = sqrt((x - spacePlanet->normal_x()) * (x - spacePlanet->normal_x()) +
             (y - spacePlanet->normal_y()) * (y - spacePlanet->normal_y()));
  } while(r < spacePlanet->gravity_range);
}

void Ship::calculate()
{
  if(phase_count < PHASE_MAX) {
    calculate_phase();
    return;
  }

  if(batt < batt_max) {
    recharge_step -= frame_time;
    if(recharge_step < 0) {
      batt += recharge_amount;
			if (batt > batt_max) batt = batt_max;
      recharge_step += recharge_rate;
    }
  }

  if(weapon_recharge > 0)
    weapon_recharge -= frame_time;
  if(special_recharge > 0)
    special_recharge -=  frame_time;

	if (!target_pressed) {
		if (target_next) {
			if (control->num_targets)
				control->set_target((control->active_target + 1) % control->num_targets);
			}
		else if (target_prev) {
			if (control->num_targets)
				control->set_target((control->active_target + control->num_targets - 1) % control->num_targets);
			}
		else if (target_close) {
			if (control->num_targets) {
				int i, j = -1;
				double r = 99999;
				double d;
				for (i = 0; i < control->num_targets; i += 1) {
					d = distance(control->target[i]);
					if (d < r) {
						r = d;
						j = i;
						}
					}
				control->set_target(j);
				}
			}
		}
	target_pressed = target_next | target_prev | target_close;

  calculate_thrust();
  calculate_turn_left();
  calculate_turn_right();
  calculate_fire_weapon();
  calculate_fire_special();

	vx += gravity_vx;
	vy += gravity_vy;
	gravity_vx = gravity_vy = 0;

  while(fabs(turn_step) > 5.625) {
    if(turn_step < 0.0) {
      angle -= 5.625;
      turn_step += 5.625;
    } else
    if(turn_step > 0.0) {
      angle += 5.625;
      turn_step -= 5.625;
    }

    if(angle < 0.0)
      angle += 360.0;
    if(angle >= 360.0)
      angle -= 360.0;
  }

  sprite_index = (int)(angle / 5.625) + 16;
	sprite_index &= 63;

  calculate_hotspots();

  SpaceObject::calculate();
}

void Ship::animate(Frame *space) {
  SpaceObject::animate(space);
	}

void Ship::animate_radar(Frame *radar) {
  SpaceObject::animate_radar(radar);
	}

void Ship::handle_damage(SpaceLocation *source) {
	batt -= fuel_sap;
	fuel_sap = 0;
	if(batt < 0) batt = 0;
	if(batt > batt_max) batt = batt_max;
	if(speed_loss > 0.0) {
		if (speed_loss < 1) throw ("Ship::handle_damage - bad speed_loss");
		speed_max *= (1.0 - (1.0 / (double)(speed_loss)));
		accel_rate *= (1.0 - (1.0 / (double)(speed_loss)));
		turn_rate *= (1.0 - (1.0 / (double)(speed_loss)));
		hotspot_rate /= (1.0 - (1.0 / (double)(speed_loss)));
		speed_loss = 0.0;
//should be revised
		}
	crew -= damage;
	damage = 0;
	crew -= directDamage;
	directDamage = 0;
	crew += repair;
	repair = 0;
	if(crew > crew_max) crew = crew_max;
	if(crew <= 0) {
		crew  = 0;
		state = 0;
		tw_sample((SAMPLE *)(melee[MELEE_BOOMSHIP].dat));
		spaceFrame->addSpaceObject(new Animation(x, y,
				kaboomSprite, KABOOM_FRAMES, time_ratio, LAYER_EXPLOSIONS));
		if (source) {
			int i, j = source->allyFlag();
			if (j == this->allyFlag()) meleeVictory = NULL;
			else if (j == -1) meleeVictory = natural_victory;
			else for (i = 0; i < num_players; i += 1) {
				if (::control[i]->ship && (::control[i]->ship->allyFlag() == j)) 
					meleeVictory = ::control[i]->ship->data->moduleVictory;
				}
			}
		}
	return;
	}

void Ship::materialize() {
	phased_in = true;
	}

void Ship::calculate_phase()
{
  if (phase_count < PHASE_MAX) {
    if(phase_step > 0)
      phase_step -= frame_time;
    else {
      phase_step += PHASE_DELAY;
      spaceFrame->addSpaceObject(new Phaser(
        x - (cos(angle * ANGLE_RATIO) * w * (double)(PHASE_MAX - phase_count)),
        y - (sin(angle * ANGLE_RATIO) * h * (double)(PHASE_MAX - phase_count)),
        this, sprite, sprite_index, hot_color, HOT_COLORS, PHASE_DELAY, LAYER_SHIPS));
      phase_count++;
      if(phase_count == PHASE_MAX)
        materialize();
    }
  }
};

void Ship::calculate_thrust() {
	if (thrust) accelerate(this, angle, accel_rate * frame_time, speed_max);
	return;
	}

void Ship::calculate_turn_left()
{
  if(turn_left) turn_step -= turn_rate * frame_time;
}

void Ship::calculate_turn_right()
{
  if(turn_right) turn_step += turn_rate * frame_time;
}

void Ship::calculate_fire_weapon()
{
  weapon_low = FALSE;

  if(fire_weapon) {
    if(batt < weapon_drain) {
      weapon_low = TRUE;
      return;
    }

    if(weapon_recharge > 0)
      return;

    if(!activate_weapon())
      return;

    batt -= weapon_drain;
    weapon_recharge += weapon_rate;

    stop_sample(data->sampleWeapon[weapon_sample]);
		tw_sample(data->sampleWeapon[weapon_sample]);
  }
}

void Ship::calculate_fire_special()
{
  special_low = FALSE;

  if(fire_special) {
    if(batt < special_drain) {
      special_low = TRUE;
      return;
    }

    if(special_recharge > 0)
      return;

    if(!activate_special())
      return;

    batt -= special_drain;
    special_recharge += special_rate;

    stop_sample(data->sampleSpecial[special_sample]);
    tw_sample(data->sampleSpecial[special_sample]);
  }
}

void Ship::calculate_hotspots() {
	if((thrust) && (hotspot_frame <= 0)) {
		spaceFrame->addSpaceObject(new Animation(
				normal_x() - (cos(angle * ANGLE_RATIO) * w / 2.5),
				normal_y() - (sin(angle * ANGLE_RATIO) * h / 2.5),
				hotspotSprite, HOTSPOT_FRAMES, time_ratio, LAYER_HOTSPOTS));
		hotspot_frame += hotspot_rate;
		}
	if (hotspot_frame > 0) hotspot_frame -= frame_time;
	return;
	}

int Ship::activate_weapon()
{
  return(TRUE);
}

inline int Ship::activate_special()
{
  return(TRUE);

}

int Ship::accelerate(SpaceLocation *source, double angle, double velocity, double max_speed) {
	double ov, nv;
	double nvx, nvy;

	if (source->isPlanet()) {
		gravity_vx += cos(angle * ANGLE_RATIO) * velocity;
		gravity_vy += sin(angle * ANGLE_RATIO) * velocity;
		return true;
		}
	if (source != this) 
		return SpaceLocation::accelerate(source, angle, velocity, max_speed);
	if (max_speed != speed_max) 
		return SpaceLocation::accelerate(source, angle, velocity, max_speed);
	ov = (vx * vx) + (vy * vy);
	nvx = vx + gravity_vx;
	nvy = vy + gravity_vy;
	nv = (nvx * nvx) + (nvy * nvy);
	if (nv < ov) {
		nvx = vx + (cos(angle * ANGLE_RATIO) * velocity) + gravity_vx*gravity_sling;
		nvy = vy + (sin(angle * ANGLE_RATIO) * velocity) + gravity_vy*gravity_sling;
		gravity_vx *= 1 - gravity_sling;
		gravity_vy *= 1 - gravity_sling;
		}
	else {
		nvx = vx + (cos(angle * ANGLE_RATIO) * velocity);
		nvy = vy + (sin(angle * ANGLE_RATIO) * velocity);
		}
	nv = (nvx * nvx) + (nvy * nvy);
	if ((ov < max_speed * max_speed) || (nv < ov)) {
		vx = nvx;
		vy = nvy;
		}
	else {
		ov = sqrt(ov);
		vx = nvx * ov / (ov + velocity);
		vy = nvy * ov / (ov + velocity);
		}
	return true;
	}

Phaser::Phaser(double ox, double oy, Ship *oship, SpaceSprite *osprite, 
							 int osprite_index, int *ocolors, int onum_colors, int ofsize, 
							 int olayer) :
		SpaceObject(ox, oy, 0.0, -1, osprite),
		ship(oship),
		sprite(osprite),
		sprite_index(osprite_index),
		colors(ocolors),
		num_colors(onum_colors),
		color_index(0),
		frame_size(ofsize),
		frame_step(ofsize)
	{
	layer = olayer;
	collide_flag_anyone = 0;
	mass = 0;
	return;
	}

void Phaser::animate(Frame *space) {
	sprite->animate_character((int)(corner_x()), (int)(corner_y()), sprite_index, 
			pallete_color[colors[color_index]], space_zoom, space);
	return;
	}

void Phaser::calculate() {
	if (!ship->exists()) state = 0;
	frame_step -= frame_time;
	while (frame_step < 0) {
		frame_step += frame_size;
		color_index++;
		if (color_index == num_colors) state = 0;
		}
	SpaceObject::calculate();
	}

