
#include <allegro.h>
#include <math.h>


#include "../melee.h"
#include "../libs.h"
#include "../ppcol.h"
#include "../frame.h"
#include "../scp.h"

#include "mmain.h"

PPCOL_MASK *pix;

void gamma_correct_bitmap(BITMAP *bmp, int gamma) {
	int x, y, w, h;
	int table[256];
	if (!gamma) return;
	for (x = 0; x < 256; x += 1) {
		table[x] = int(255.5 * pow(x / 255., 1-gamma/300.));
		}
	w = bmp->w;
	h = bmp->h;
	for (y = 0; y < h; y += 1) {
		for (x = 0; x < w; x += 1) {
			int r, g, b;
			b = getpixel(bmp, x, y);
			if (b == bitmap_mask_color(bmp)) continue;
			r = getr(b);
			g = getg(b);
			b = getb(b);
			r = table[r];
			g = table[g];
			b = table[b];
			putpixel(bmp, x, y, makecol(r, g, b));
			}
		}
	return;
	}
void trans_correct_bitmap(BITMAP *bmp) {
	if (bitmap_color_depth(bmp) != 8) return;
	int w, h;
	w = bmp->w;
	h = bmp->h;
	for (int y = 0; y < h; y += 1) {
		for (int x = 0; x < w; x += 1) {
			int c = _getpixel(bmp, x, y);
			if ((getr8(c) == 255) && (getb8(c) == 255) && (getg8(c) == 0)) 
					_putpixel(bmp, x, y, MASK_COLOR_8);
			}
		}
	return;
	}

SpaceSprite::SpaceSprite(DATAFILE *sprites, int sprite_count)
{
  int i;

  count = sprite_count;

  m = new PPCOL_MASK*[count];
  b = new BITMAP*[count];

  w = ((RLE_SPRITE *)sprites[0].dat)->w;
  h = ((RLE_SPRITE *)sprites[0].dat)->h;

  for(i = 0; i < count; i++) {
    b[i] = create_bitmap(w, h);
    clear_to_color(b[i], makecol(255,0,255));
    draw_rle_sprite(b[i], (RLE_SPRITE *)(sprites[i].dat), 0, 0);
		trans_correct_bitmap(b[i]);
		gamma_correct_bitmap(b[i], gamma_correction);
    m[i]                 = create_ppcol_mask(b[i]);

  }
	
}

SpaceSprite::SpaceSprite(BITMAP **bmp, int bmp_count)
{
  int i;

  count = bmp_count;

  m = new PPCOL_MASK*[count];
  b = new BITMAP*[count];

  w = bmp[0]->w;
  h = bmp[0]->h;


  for(i = 0; i < count; i++) {
    b[i] = create_bitmap(w, h);
    clear_to_color(b[i], makecol(255,0,255));
    draw_sprite(b[i], bmp[i], 0, 0);
		trans_correct_bitmap(b[i]);
		gamma_correct_bitmap(b[i], gamma_correction);
    m[i]                 = create_ppcol_mask(b[i]);

  }

}

SpaceSprite::~SpaceSprite()
{
  int i;
  
  for(i = 0; i < count; i++) {
    destroy_ppcol_mask(m[i]);
  }

  for(i = 0; i < count; i++) {
    destroy_bitmap(b[i]);
  }
}

BITMAP *SpaceSprite::get_bitmap(int index)
{
  return(b[index]);
}

void SpaceSprite::animate(int x, int y, int index, double zoom, Frame *space)
{
  int spr_w = int(double(w) * zoom);
  int spr_h = int(double(h) * zoom);
  
  if(spr_w < 1) spr_w = 1;
  if(spr_h < 1) spr_h = 1;


  if (antia_on) {
    aa_stretch_sprite(space->frame, b[index], x, y, spr_w, spr_h);
  } else {
    stretch_sprite(space->frame, b[index], x, y, spr_w, spr_h);
  }
  
  space->add_box(x, y, spr_w, spr_h);
}

void SpaceSprite::draw_character(int x, int y, int index, int color,
  BITMAP *bmp)
{
  PPCOL_MASK *mask = m[index];

#ifdef SLOW_SPRITE
  int i,j;
  for(i = 0; i < mask->h; i++)
    for(j = 0; j < mask->w; j++)
      if( check_ppmask_collision(mask, pix, 0, 0, i, j) )
        putpixel(bmp, x + i, y + j, color);
#else
	int row, chunk;
	for(row = 0; row < mask->h; row++) {
		for(chunk = 0; chunk <= mask->max_chunk; chunk++) {
			int c = mask->sp_mask[row][chunk];
			int cx = x + (chunk << 5);
			while (c) {
				if (c & 0x80000000) putpixel(bmp, cx, y + row, color);
				c = c << 1;
				cx += 1;
				}
			}
		}
#endif
	}

void SpaceSprite::draw_character_stretch(int x, int y, int index, int color,double zoom,
		BITMAP *bmp)
{
  PPCOL_MASK *mask = m[index];

#ifdef SLOW_SPRITE
  int i, j; 
  for(i = 0; i < mask->h; i++)
    for(j = 0; j < mask->w; j++)
      if( check_ppmask_collision(mask, pix, 0, 0, i, j) ) 
        putpixel(bmp, x + int(i * zoom), y + int(j * zoom), color);
#else
	int zoom_factor;
	const int ebits = 8;
	int         row, chunk, cx, ry;
	zoom_factor = int(zoom * (1 << (ebits)));
	for(row = 0; row < mask->h; row++) {
		cx = x << ebits;
		ry = y + ((row * zoom_factor) >> ebits);
		for(chunk = 0; chunk <= mask->max_chunk; chunk++) {
			int c = mask->sp_mask[row][chunk];
			cx = (x << ebits) + chunk * (zoom_factor << 5);
			while (c) {
				if (c & 0x80000000) putpixel(bmp, cx >> ebits, ry, color);
				c = c << 1;
				cx += zoom_factor;
				}
			}
		}
#endif	
	}

void SpaceSprite::animate_character(int x, int y, int index, int color, 
																		double zoom, Frame *space)
{
	int spr_w = int( double(w) * zoom)+2;
	int spr_h = int( double(h) * zoom)+2;

	if(spr_w < 1) spr_w = 1;
	if(spr_h < 1) spr_h = 1;

	draw_character_stretch(x,y,index,color,zoom,space->frame);

	space->add_box(x, y, spr_w, spr_h);
}

int SpaceSprite::collide(int x, int y, int i, int ox, int oy, int oi,
		SpaceSprite *other)
{
  return(check_ppmask_collision(m[i], other->m[oi], x, y, ox, oy));
}

int line_collide;
int rect_x;
int rect_y;
int rect_w;
int rect_h;
int collide_x;
int collide_y;

PPCOL_MASK *rect_mask;

void check_line_collision(BITMAP *bmp, int x, int y, int d)
{
  int row, col;

  if(line_collide)
    return;

  col = x - rect_x;
  row = y - rect_y;

#ifdef SLOW_SPRITE
  if((col >= 0) && (col < rect_w) && (row >= 0) && (row < rect_h)) 
    if(check_ppmask_collision(rect_mask, pix, 0, 0, col, row)) {
      line_collide = TRUE;
      collide_x = x;
      collide_y = y;
    }
#else
  if((col >= 0) && (col < rect_w) &&
     (row >= 0) && (row < rect_h)) {
    if((rect_mask->sp_mask[row][col >> 5] & (0x80000000 >> (col & 31))) != 0) {
      line_collide = TRUE;
      collide_x = x;
      collide_y = y;
    }
  }
#endif
}

int SpaceSprite::collide_line(int lx1, int ly1, int *lx2, int *ly2,
  int sx, int sy, int sindex)
{
  line_collide = FALSE;
  rect_x = sx - (w / 2);
  rect_y = sy - (h / 2);
  rect_w = w;
  rect_h = h;
	if ((rect_x   > lx1) && (rect_x   > *lx2)) return FALSE;
	if ((rect_x+w < lx1) && (rect_x+w < *lx2)) return FALSE;
	if ((rect_y   > ly1) && (rect_y   > *ly2)) return FALSE;
	if ((rect_y+h < ly1) && (rect_y+h < *ly2)) return FALSE;
  rect_mask = m[sindex];
  do_line(NULL, lx1, ly1, *lx2, *ly2, 0, check_line_collision);
  if(line_collide) {
    *lx2 = collide_x;
    *ly2 = collide_y;
    return(TRUE);
  }

  return(FALSE);
}

 int SpaceSprite::frames()
{
  return(count);
}

 double SpaceSprite::width()
{
  return((double)(w));
}

 double SpaceSprite::height()
{
  return((double)(h));
}

SpaceRotatingSprite::SpaceRotatingSprite(DATAFILE *sprites, int sprite_count) :
  SpaceSprite(*this)
{
  int i, j;
  BITMAP *bmp, *bmp1;

  count = sprite_count;

  m = new PPCOL_MASK*[count * 64];
  b = new BITMAP*    [count * 64];

  w = ((RLE_SPRITE *)sprites[0].dat)->w;
  h = ((RLE_SPRITE *)sprites[0].dat)->h;

  bmp  = create_bitmap(w, h);

  for(i = 0; i < count; i++) {
    clear_to_color(bmp, makecol(255,0,255));
		draw_rle_sprite(bmp, (RLE_SPRITE *)(sprites[i].dat), 0, 0);
		trans_correct_bitmap(bmp);
		gamma_correct_bitmap(bmp, gamma_correction);
    for(j = 0; j < 64; j++) {
      bmp1 = create_bitmap(w, h);
			clear_to_color(bmp1, bitmap_mask_color(bmp1));
	    rotate_sprite(bmp1, bmp, 0, 0, itofix(4 * j));
      m[(i * 64) + j]                 = create_ppcol_mask(bmp1);
      b[(i * 64) + j]=bmp1;
    }
  }

  count *= 64;

  destroy_bitmap(bmp);
}
