#include "../ship.h"

class ClairconctlarPinnacle : public Ship {
  double       weaponRange;
  double       weaponVelocity;
  int          weaponDamage;
  int          weaponArmour;

  double       specialVelocity;
  int          specialArmour;
  int          specialFired;
  Shot        *specialObject;
  int          specialSoundCount;
  int          beep;

  public:
  ClairconctlarPinnacle(double ox, double oy, double shipAngle,
    int shipCollideFlag, int shipRadarColor, ShipData *shipData, Control *cntrl);
  void set_beep (int set);
  int isBad_Warp(SpaceObject *o);
  virtual int activate_weapon();
  virtual int activate_special();
  virtual void calculate();
};

class ClairconctlarBeacon : public Shot {

  ClairconctlarPinnacle *mother;

  public:
  ClairconctlarBeacon(double ox, double oy, double oangle, double ov, int odamage,
    double orange, int oarmour, Ship *oship, SpaceSprite *osprite,
    ClairconctlarPinnacle *opinnacle);

  virtual void inflict_damage(SpaceObject *other);
  void soundExplosion();

};

ClairconctlarPinnacle::ClairconctlarPinnacle(double ox, double oy, double shipAngle,
  int shipCollideFlag, int shipRadarColor, ShipData *shipData, Control *cntrl) :
  Ship(ox, oy, shipAngle, shipCollideFlag, shipRadarColor, shipData, cntrl)
{

  collide_flag_ally = bit(LAYER_SHOTS);

  weaponRange    = scale_range(get_config_float("Weapon", "Range", 0));
  weaponDamage   = get_config_int("Weapon", "Damage", 0);
  weaponArmour   = get_config_int("Weapon", "Armour", 0);
  weaponVelocity = scale_velocity(get_config_int("Weapon", "Velocity", 0));

  specialVelocity    = scale_velocity(get_config_float("Special", "Velocity", 0));
  specialArmour      = get_config_int("Special", "Armour", 0);
  specialObject      = NULL;
  specialFired       = FALSE;
  specialSoundCount  = 0;
  beep               = FALSE;
}

void ClairconctlarPinnacle::set_beep(int set)
{
  beep = set;
}

int ClairconctlarPinnacle::activate_weapon()
{
  spaceFrame->addSpaceObject(new AnimatedShot(
    0.0, (height() / 2.0), angle, weaponVelocity, weaponDamage, weaponRange,
    weaponArmour, this, this, data->spriteWeapon, 10, 1 ));
  spaceFrame->addSpaceObject(new AnimatedShot(
    -(width() / 3.0), (height()/20.0), angle - 90.0, weaponVelocity, weaponDamage,
    weaponRange, weaponArmour, this, this, data->spriteWeapon, 10, 1 ));
  spaceFrame->addSpaceObject(new AnimatedShot(
    width() / 3.0,  -(height() / 8.0), angle + 90.0, weaponVelocity, weaponDamage,
    weaponRange, weaponArmour, this, this, data->spriteWeapon, 10, 1 ));
  spaceFrame->addSpaceObject(new AnimatedShot(
    -(width() / 3.0), -(height() / 8.0), angle - 90.0, weaponVelocity, weaponDamage,
    weaponRange, weaponArmour, this, this, data->spriteWeapon, 10, 1 ));
  spaceFrame->addSpaceObject(new AnimatedShot(
    width() / 3.0, (height()/20.0), angle + 90.0, weaponVelocity, weaponDamage,
    weaponRange, weaponArmour, this, this, data->spriteWeapon, 10, 1 ));

  return(TRUE);

}

int ClairconctlarPinnacle::activate_special()
{
  if(specialFired)
    return(FALSE);
  specialObject = new ClairconctlarBeacon(0.0, -(height() / 2.0), angle + 180.0,
    specialVelocity, 0, -1.0, specialArmour, this, data->spriteSpecial, this);
  specialSoundCount = 0;
  beep = TRUE;
  spaceFrame->addSpaceObject(specialObject);
  specialFired = TRUE;
  return(TRUE);
}

void ClairconctlarPinnacle::calculate()
{
  double ox,oy,odx,ody,oangle;
  int increaseX,increaseY;
  Ship::calculate();
  if((specialObject != NULL) && (!specialObject->exists())) {
    stop_sample(data->sampleSpecial[1]);
    specialObject = NULL; }
  if(specialFired && (!fire_special))
    specialFired = FALSE;
  if((specialFired) && (fire_special) && (beep)) {
    specialSoundCount += frame_time;
    if (specialSoundCount >= 2500) {
       specialSoundCount = 0;
       stop_sample(data->sampleSpecial[1]);
       tw_sample(data->sampleSpecial[1]);}
    }
  if((specialObject != NULL) && (!fire_special)) {
    beep = FALSE;
    ox=specialObject->normal_x();
    oy=specialObject->normal_y();
    odx=specialObject->get_vx();
    ody=specialObject->get_vy();
    oangle=specialObject->get_angle();
    specialObject->destroy();
    x=(ox-(height()/2.0));
    y=(oy-(height()/2.0));
    vx=odx;
    vy=ody;
    angle=oangle;
       if (isBad_Warp((SpaceObject*)spacePlanet))
  {
    crew  = 0;
    x = spacePlanet->normal_x();
    y = spacePlanet->normal_y();
    state = 0;
 } else
  if (target)
   if (isBad_Warp((SpaceObject*)target))
    {
      increaseX =(x > target->normal_x());
      increaseY =(y > target->normal_y());
      x=target->normal_x();
      y=target->normal_y();    

      if (increaseX) {
      if (w > target->width())
       x+=w;
      else
       x+=target->width();
      } else {
      if (w > target->width())
       x-=w;
      else
       x-=target->width();      
      }
      if (increaseY) {
      if (h > target->height()) 
       y+=h;
      else
       y+=target->height();
      } else {
      if (h > target->height())
       y-=h;
      else
       y-=target->height();
      }
    }
    specialObject = NULL;
    stop_sample(data->sampleSpecial[1]);
    stop_sample(data->sampleExtra[0]);
    tw_sample(data->sampleExtra[0]);
  }
}

ClairconctlarBeacon::ClairconctlarBeacon(double ox, double oy, double oangle,
  double ov, int odamage, double orange, int oarmour, Ship *oship,
  SpaceSprite *osprite, ClairconctlarPinnacle *omother) :
  Shot (ox, oy, oangle, ov, odamage, orange, oarmour, oship, oship, osprite)
  {
    mother = omother;
  }

void ClairconctlarBeacon::soundExplosion()
{
  stop_sample(ship->data->sampleSpecial[1]);
  mother->set_beep(FALSE);
}

void ClairconctlarBeacon::inflict_damage(SpaceObject *other)
{
  stop_sample(ship->data->sampleSpecial[1]);
  mother->set_beep(FALSE);
  Shot::inflict_damage(other);
}

int ClairconctlarPinnacle::isBad_Warp(SpaceObject *o)
{
 double _d = distance_from(o->normal_x(), o->normal_y(), 
                              normal_x(), normal_y());

  if (_d <= w * 1.8)  //too dirty, but enough for planet
   return (TRUE);

 return (FALSE);
}

Ship *getClairconctlarPinnacle(double sX, double sY, double sAngle,
  int sCollideFlag, int sRadarColor, ShipData *sData, Control *cntrl)
{
  return(new ClairconctlarPinnacle(sX, sY, sAngle, sCollideFlag, sRadarColor, sData, cntrl));
}

static ShipRegistration ignore_me(
	"clapi",
	"Clairconctlar Pinnacle",
	getClairconctlarPinnacle
	);
