
#include "../ship.h"

class KohrAhBlade;

class KohrAhMarauder : public Ship
{
  double       weaponRange;
  double       weaponVelocity;
  int          weaponDamage;
  int          weaponArmour;
  int          weaponFired;
  KohrAhBlade **weaponObject;

  double       specialVelocity;
  int          specialDamage;
  int          specialArmour;
  double       specialRange;

  public:
  KohrAhMarauder(double ox, double oy, double shipAngle,
    int shipCollideFlag, int shipRadarColor, ShipData *shipData, Control *cntrl);

  virtual int activate_weapon();
  virtual int activate_special();
  virtual void calculate();

  int numblades;
  int maxblades;
};

class KohrAhBlade : public AnimatedShot {
  int       passive;
  double    passiveRange;
  double    passiveVelocity;

  public:
  KohrAhBlade(double ox, double oy, double oangle, double ov, int odamage,
    double orange, int oarmour, Ship *oship,
    SpaceSprite *osprite, int ofcount, int ofsize);

  virtual void calculate();
  virtual void animateExplosion();

  void disengage();
};

class KohrAhBladeDecay : public Animation {
  public:
  KohrAhBladeDecay(double ox, double oy, double ovx, double ovy,
    SpaceSprite *osprite, int ofcount, int ofsize);
};

class KohrAhFRIED : public Shot {
  int frame_count;

  public:
  KohrAhFRIED(double oangle, double ov, int odamage, double orange,
    int oarmour, Ship *oship, SpaceSprite *osprite, int ofcount);

  virtual void calculate();
	virtual void inflict_damage(SpaceObject *other);
};

void KohrAhFRIED::inflict_damage(SpaceObject *other) {
	if ((other->getID() & ~SPACE_MASK) == CHMMR_SPEC) other->state = 0;
	Shot::inflict_damage(other);
	return;
	}

KohrAhMarauder::KohrAhMarauder(double ox, double oy, double shipAngle,
		int shipCollideFlag, int shipRadarColor, ShipData *shipData, Control *cntrl) :
		Ship(ox, oy, shipAngle, shipCollideFlag, shipRadarColor, shipData, cntrl) {
	weaponRange    = scale_range(get_config_float("Weapon", "Range", 0));
	weaponVelocity = scale_velocity(get_config_float("Weapon", "Velocity", 0));
	weaponDamage   = get_config_int("Weapon", "Damage", 0);
	weaponArmour   = get_config_int("Weapon", "Armour", 0);
	weaponFired    = 0;

	specialRange    = scale_range(get_config_float("Special", "Range", 0));
	specialVelocity = scale_velocity(get_config_float("Special", "Velocity", 0));
	specialDamage   = get_config_int("Special", "Damage", 0);
	specialArmour   = get_config_int("Special", "Armour", 0);

	numblades = 0;
	maxblades = get_config_int("Weapon", "MaxBlades", 0);
	weaponObject = new KohrAhBlade*[maxblades];
	for (int i = 0; i < maxblades; i += 1) {
		weaponObject[i] = NULL;
		}
	}

int KohrAhMarauder::activate_weapon() {
	if (weaponFired) return(FALSE);
	if (numblades == maxblades) {
		weaponObject[0]->state = 0;
		numblades -= 1;
		for (int i = 0; i < numblades; i += 1) {
			weaponObject[i] = weaponObject[i + 1];
			}
		weaponObject[numblades] = NULL;
		}
	weaponObject[numblades] = new KohrAhBlade(0.0, (h / 2.0), angle, weaponVelocity,
			weaponDamage, weaponRange, weaponArmour, this, data->spriteWeapon,
			10, 40);
	spaceFrame->addSpaceObject(weaponObject[numblades]);
	numblades += 1;
	weaponFired = 1;
	return(TRUE);
	}

int KohrAhMarauder::activate_special() {
	int i;
	for(i = 0; i < 16; i++) {
		spaceFrame->addSpaceObject(new KohrAhFRIED(
				((double)(i) * 22.5) - 180.0, specialVelocity, specialDamage,
				specialRange, specialArmour, this, data->spriteSpecial, 20));
		}
	return(TRUE);
	}

void KohrAhMarauder::calculate() {
	Ship::calculate();
	if (!fire_weapon) {
		if(weaponFired == 1) {
			weaponObject[numblades-1]->disengage();
			}
		weaponFired = 0;
		}
	else if ((weaponFired == 1) && !weaponObject[numblades-1]->exists()) {
		weaponFired = 2;
		}
	int j = 0;
	for (int i = 0; i < numblades; i += 1) {
		weaponObject[i-j] = weaponObject[i];
		if (!weaponObject[i]->exists()) j += 1;
		if (j) weaponObject[i] = NULL;
		}
	numblades -= j;		
	return;
	}

KohrAhBlade::KohrAhBlade(double ox, double oy, double oangle, double ov,
  int odamage, double orange, int oarmour, Ship *oship,
  SpaceSprite *osprite, int ofcount, int ofsize) :
  AnimatedShot(ox, oy, oangle, ov, odamage, -1.0, oarmour, oship, oship,
    osprite, ofcount, ofsize),
  passive(FALSE),
  passiveRange(orange)
{
  explosionSprite     = ship->data->spriteWeaponExplosion;
  explosionFrameCount = 20;
  explosionFrameSize  = 40;
}

void KohrAhBlade::calculate() {
	SpaceObject *o, *t = NULL;
	double oldrange = 999999;

	AnimatedShot::calculate();

	if(passive) {
		Query a;
		for (a.begin(this, bit(LAYER_SHIPS) + bit(LAYER_SHOTS) + bit(LAYER_SPECIAL), 
				passiveRange); a.current; a.next()) {
			o = a.currento;
			if (!o->isAlly(this) && (distance(o) < oldrange) && (o->allyFlag() != -1)) {
				t = o;
				oldrange = distance(o);
				}
			}
		if (t) {
//			angle = trajectory_angle(t);
//			vx = (v / 10.0) * (cos(angle * ANGLE_RATIO));
//			vy = (v / 10.0) * (sin(angle * ANGLE_RATIO));
			vx = (v / 10.0 / oldrange) * 
				(min_delta(t->normal_x(), normal_x(), X_MAX));
			vy = (v / 10.0 / oldrange) * 
				(min_delta(t->normal_y(), normal_y(), Y_MAX));
			}
		else {
			vx = 0;
			vy = 0;
			}
		}
	return;
	}

void KohrAhBlade::animateExplosion()
{
  spaceFrame->addSpaceObject(new KohrAhBladeDecay(x, y,
    vx, vy, explosionSprite, explosionFrameCount, explosionFrameSize));
}

void KohrAhBlade::disengage()
{
  passive = TRUE;
}

KohrAhBladeDecay::KohrAhBladeDecay(double ox, double oy, double ovx,
  double ovy, SpaceSprite *osprite, int ofcount, int ofsize) :
  Animation(ox, oy, osprite, ofcount, ofsize, LAYER_EXPLOSIONS)
{
  vx = ovx;
  vy = ovy;
}

KohrAhFRIED::KohrAhFRIED(double oangle, double ov, int odamage,
  double orange, int oarmour, Ship *oship, SpaceSprite *osprite,
  int ofcount) :
  Shot(0.0, 0.0, oangle, ov, odamage, orange, oarmour, oship, oship, osprite, 1.0),
  frame_count(ofcount)
{
  x += cos(angle * ANGLE_RATIO) * 50.0;
  y += sin(angle * ANGLE_RATIO) * 50.0;
}

void KohrAhFRIED::calculate() {
  sprite_index = (int)((d / range) * (double)(frame_count - 1));
  Shot::calculate();
}

Ship *getKohrAhMarauder(double sX, double sY, double sAngle,
  int sCollideFlag, int sRadarColor, ShipData *sData, Control *cntrl)
{
  return(new KohrAhMarauder(
    sX, sY, sAngle, sCollideFlag, sRadarColor, sData, cntrl));
}

static ShipRegistration ignore_me(
  "kohma",
  "Kohr-Ah Marauder",
  getKohrAhMarauder
	);
