
#include "../ship.h"

class MelnormeTrader : public Ship {
  double       weaponRange;
  double       weaponVelocity;
  int          weaponDamage;
  int          weaponArmour;
  SpaceObject *weaponObject;

  double       specialRange;
  double       specialVelocity;
  int          specialFrames;
  int          specialArmour;

  public:
  MelnormeTrader(double ox, double oy, double shipAngle, int shipCollideFlag,
    int shipRadarColor, ShipData *shipData, Control *cntrl);

  virtual void calculate();

  virtual int activate_weapon();
  virtual int activate_special();
};

class MelnormeShot : public Shot {
  double v;
  Ship* anime;
  int frame;
  int frame_step;
  int frame_size;
  int frame_count;
  int charge_frame;
  int charge_phase;
  int released;

  public:
  MelnormeShot(double ox, double oy, double oangle, double ov, int odamage,
    double orange, int oarmour, Ship *oship, SpaceSprite *osprite,
    int ofcount, int ofsize);

  virtual void calculate();
  virtual void animateExplosion();
  virtual void handle_damage(SpaceLocation *source);
};

class MelnormeShotExplosion : public Animation {
  public:
  MelnormeShotExplosion(double ox, double oy, SpaceSprite *osprite,
    int last_frame, int frame_ticks, int first_frame);
};

class MelnormeDisable : public SpaceObject {
  Ship *ship;
  int   disableframe;
  int   disableframe_count;
  int   frame_step;
  int   frame_size;
  int   frame_count;

  public:
  MelnormeDisable(Ship *oship, SpaceSprite *osprite, int ofcount,
    int ofsize, int disableFrames);

  virtual void calculate();
};

class MelnormeSpecial : public AnimatedShot {
  int disableFrames;
  SpaceSprite *disableSprite;

  public:
  MelnormeSpecial(double ox, double oy, double oangle, double ov,
    int oframes, double orange, int oarmour, Ship *oship,
    SpaceSprite *osprite, int ofsize, int ofcount);

  void inflict_damage(SpaceObject *other);
  void animateExplosion();
};

MelnormeTrader::MelnormeTrader(double ox, double oy, double shipAngle,
  int shipCollideFlag, int shipRadarColor, ShipData *shipData, Control *cntrl) :
  Ship(ox, oy, shipAngle, shipCollideFlag, shipRadarColor, shipData, cntrl)
{

  weaponRange    = scale_range(get_config_float("Weapon", "Range", 0));
  weaponVelocity = scale_velocity(get_config_float("Weapon", "Velocity", 0));
  weaponDamage   = get_config_int("Weapon", "Damage", 0);
  weaponArmour   = get_config_int("Weapon", "Armour", 0);
  weaponObject   = NULL;

  specialRange    = scale_range(get_config_float("Special", "Range", 0));
  specialVelocity = scale_velocity(get_config_float("Special", "Velocity", 0));
  specialFrames   = get_config_int("Special", "Frames", 0);
  specialArmour   = get_config_int("Special", "Armour", 0);
}

void MelnormeTrader::calculate()
{
  Ship::calculate();
  if (weaponObject) {
    if (!weaponObject->exists()) weaponObject = NULL;
    if (fire_weapon) recharge_step += frame_time; else weaponObject = NULL;
    }
}

int MelnormeTrader::activate_weapon()
{
  if(weaponObject)
    return(FALSE);
  weaponObject = spaceFrame->addSpaceObject(new MelnormeShot(
    0.0, (height() / 2.0), angle, weaponVelocity, weaponDamage, weaponRange,
    weaponArmour, this, data->spriteWeapon, 10, 50));
  return(TRUE);
}

int MelnormeTrader::activate_special() {
  spaceFrame->addSpaceObject( new MelnormeSpecial(
    0.0, (height() / 2.0), angle, specialVelocity, specialFrames,
    specialRange, specialArmour, this, data->spriteSpecial, 20, 50));
  return(TRUE);
}

MelnormeShot::MelnormeShot(double ox, double oy, double oangle, double ov,
  int odamage, double orange, int oarmour, Ship *oship, SpaceSprite *osprite,
  int ofcount, int ofsize) :
  Shot(ox, oy, oangle, ov, odamage, orange, oarmour, oship, oship, osprite),
  v(ov),
  frame(0),
  frame_step(0),
  frame_size(ofsize),
  frame_count(ofcount),
  charge_frame(0),
  charge_phase(0),
  released(FALSE),
  anime(oship)
{
  vx = 0.0;
  vy = 0.0;
}

void MelnormeShot::calculate()
{
	if(released)
		Shot::calculate();
	else {
//		SpaceObject::calculate();
		if(!ship->exists()) {
			state = 0;
//			ship = NULL;
			}
		}
	frame_step += frame_time;
	if(frame_step >= frame_size) {
		frame_step -= frame_size;
		frame++;
		if(frame == frame_count) {
			frame = 0;
			if((!released) && (charge_phase < 3)) {
				charge_frame++;
				if(charge_frame == 5) {
					charge_frame = 0;
					charge_phase++;
					damage_factor *= 2;
					armour *= 2;
					tw_sample(ship->data->sampleWeapon[0]);
					}
				}
			}
		}
	sprite_index = (charge_phase * 10) + frame;
	if(!released) {
		angle = ship->get_angle();
		x = ship->normal_x() + (cos(angle * ANGLE_RATIO) * (ship->width() / 2.0));
		y = ship->normal_y() + (sin(angle * ANGLE_RATIO) * (ship->height() / 2.0));
		x += frame_time * ship->get_vx();
		y += frame_time * ship->get_vy();
		}
	if((!released) && (!ship || !ship->fire_weapon)) {
		vx = cos(angle * ANGLE_RATIO) * v;
		vy = sin(angle * ANGLE_RATIO) * v;
		released = TRUE;
		tw_sample(anime->data->sampleSpecial[0]);
		}
	}

void MelnormeShot::animateExplosion()
{
  spaceFrame->addSpaceObject( new MelnormeShotExplosion(x, y,
    anime->data->spriteWeaponExplosion, (charge_phase * 20) + 20, 20,
    charge_phase * 20));
}

void MelnormeShot::handle_damage(SpaceLocation *source)
{
  int old = armour;
  Shot::handle_damage(source);
  if (!released && (armour > 0)) armour = old;
}
    
MelnormeShotExplosion::MelnormeShotExplosion(double ox, double oy,
		SpaceSprite *osprite, int last_frame, int frame_ticks, int first_frame) :
		Animation(ox, oy, osprite, last_frame, frame_ticks, LAYER_EXPLOSIONS) {
	sprite_index = first_frame;
	}

MelnormeDisable::MelnormeDisable(Ship *oship, SpaceSprite *osprite,
  int ofcount, int ofsize, int disableFrames) :
  SpaceObject(oship->normal_x(), oship->normal_y(), 0.0, -1, osprite),
  ship(oship),
  disableframe(0),
  disableframe_count(disableFrames),
  frame_step(0),
  frame_size(ofsize),
  frame_count(ofcount)
{
	collide_flag_anyone = 0;
}

void MelnormeDisable::calculate() {
	frame_step+= frame_time;
	while (frame_step >= frame_size) {
		frame_step -= frame_size;
		sprite_index++;
		if(sprite_index == frame_count)
			sprite_index = 0;
		}
	if(!ship->exists()) {
		state = 0;
		return;
		}
	x = ship->normal_x();
	y = ship->normal_y();
	ship->nextkeys &= ~(keyflag::left | keyflag::right | keyflag::special);
	ship->nextkeys |= keyflag::right;
	disableframe += frame_time;
	if (disableframe >= disableframe_count) state = 0;
	SpaceObject::calculate();
	}

MelnormeSpecial::MelnormeSpecial(double ox, double oy, double oangle,
  double ov, int oframes, double orange, int oarmour, Ship *oship,
  SpaceSprite *osprite, int ofcount, int ofsize) :
  AnimatedShot(ox, oy, oangle, ov, 0, orange, oarmour, oship, oship, osprite,
    ofcount, ofsize),
  disableFrames(oframes),
  disableSprite(oship->data->spriteExtra)
{
	collide_flag_anyone = bit(LAYER_SHIPS);
}

void MelnormeSpecial::animateExplosion() {}

void MelnormeSpecial::inflict_damage(SpaceObject *other) {
	if (other->isShip()) spaceFrame->addSpaceObject(new MelnormeDisable(
			(Ship *)(other), disableSprite, 20, 50, disableFrames));
	state = 0;
	return;
	}

Ship *getMelnormeTrader(double sX, double sY, double sAngle,
  int sCollideFlag, int sRadarColor, ShipData *sData, Control *cntrl)
{
  return(new MelnormeTrader(
    sX, sY, sAngle, sCollideFlag, sRadarColor, sData, cntrl));
}

static ShipRegistration ignore_me(
  "meltr",
  "Melnorme Trader",
  getMelnormeTrader
	);
