
#include "../ship.h"

class SlylandroProbe : public Ship {
  int frame;
  int thrustActive;
  int thrustForward;

	int sprite_index2;

  public:
  SlylandroProbe(double ox, double oy, double shipAngle, int shipCollideFlag,
    int shipRadarColor, ShipData *shipData, Control *cntrl);

  int crewPanelColor();

  virtual void calculate_thrust();
  virtual void calculate_hotspots();
  virtual int activate_weapon();
  virtual void calculate_fire_special();
  virtual void calculate();
};

class SlylandroLaser : public Laser {
	int level;
	int fired;
	double old_length;
	SpaceLocation *target;
	Ship *ship;

  public:
  SlylandroLaser(Ship *lship, SpaceLocation *lroot, SpaceLocation *ltarget, 
			int llevel);

  virtual void calculate();
};

SlylandroProbe::SlylandroProbe(double ox, double oy, double shipAngle,
  int shipCollideFlag, int shipRadarColor, ShipData *shipData, Control *cntrl) :
  Ship(ox, oy, shipAngle, shipCollideFlag, shipRadarColor, shipData, cntrl),
  frame(0),
  thrustActive(FALSE),
  thrustForward(TRUE)
{
	sprite_index2 = random() & 63;
}

int SlylandroProbe::crewPanelColor()
{
  return pallete_color[8];
}

void SlylandroProbe::calculate()
{
	Ship::calculate();

  frame+= frame_time;
  if(frame >= 50) {
    frame -= 50;
    if(thrustForward) {
      sprite_index2++;
      if(sprite_index2 == 64)
        sprite_index2 = 0;
    } else {
      sprite_index2--;
      if(sprite_index2 == -1)
        sprite_index2 = 63;
    }
  }
	sprite_index = sprite_index2;

}

void SlylandroProbe::calculate_hotspots() {
	}
void SlylandroProbe::calculate_thrust() {
	if (thrust && !thrustActive) {
		angle = angle + 180.0;
		if (angle > 360.0) angle -= 360.0;
		}
	thrustActive = thrust;
	thrust = true;
	Ship::calculate_thrust();
	thrust = thrustActive;
	return;
	}

int SlylandroProbe::activate_weapon()
{

	SpaceLocation *t = NULL;
	double r = 99999;
	int i;
	for (i = 0; i < control->num_targets; i += 1) {
		if (distance(control->target[i]) < r) {
			t = control->target[i];
			r = distance(t);
			}
		}
	spaceFrame->addSpaceLine(new SlylandroLaser( this, this, t, 4));

  return(TRUE);
}

void SlylandroProbe::calculate_fire_special()
{
  int fire = FALSE, i;
  SpaceObject *o;

  special_low = FALSE;

  if(fire_special) {
    if(special_recharge != 0)
      return;

    for(i = 0; (i < SPACE_LAYERS) && (!fire); i++) {
      o = spaceFrame->getFirstObject(i);
      while((o != NULL) && (!fire)) {
        if((o->getID() == SPACE_ASTEROID) && (distance(o) < w)) {
          damage_factor = 1;
          inflict_damage(o);
          damage_factor = 0;
          batt = batt_max;
          fire = TRUE;
        }
        o = o->nexto;
      }
    }

    if(!fire)
      return;

    special_recharge = special_rate;

    stop_sample(data->sampleSpecial[0]);
    tw_sample(data->sampleSpecial[0]);
  }
}

SlylandroLaser::SlylandroLaser(Ship *lship, SpaceLocation *lroot, 
		SpaceLocation *ltarget, int llevel) :
		Laser(0, pallete_color[15], 0.0, 1, 200, lroot, 0.0, 0.0),
		level(llevel), 
		fired(0),
		target(ltarget),
		ship(lship) {
	old_length = length = (double) (random() % 90);
	if (level <= 0) state = 0;

	x = pos->normal_x();
	y = pos->normal_y();
	if (target) angle = trajectory_angle(target);
	else angle = random() % 360;
	if (ship != pos) {
		x += ((SlylandroLaser *)(pos))->edge_x();
		y += ((SlylandroLaser *)(pos))->edge_y();
		angle += (random() % 60) - 30;
		}
	color = pallete_color[79 - level * 2 - (random()%8)];
	}

void SlylandroLaser::calculate() {
	Laser::calculate();
	if (!fired && (length == old_length)) {
		for (int i = 0; i < 3; i += 1) {
			if ((random() % 5) - i > 0) {
				spaceFrame->addSpaceLine(new SlylandroLaser( ship, this, target, level-1));
				}
			}
		fired = 1;
		}
	if (!pos->exists()) return;
		x = pos->normal_x();
		y = pos->normal_y();
	if (ship != pos) {
		x += ((SlylandroLaser *)(pos))->edge_x();
		y += ((SlylandroLaser *)(pos))->edge_y();
		}
	return;
	}

Ship *getSlylandroProbe(double sX, double sY, double sAngle,
  int sCollideFlag, int sRadarColor, ShipData *sData, Control *cntrl)
{
  return(new SlylandroProbe(
    sX, sY, sAngle, sCollideFlag, sRadarColor, sData, cntrl));
}

static ShipRegistration ignore_me(
  "slypr",
  "Slylandro Probe",
  getSlylandroProbe
	);
