
#include "../ship.h"

class VuxIntruder : public Ship {
  int    weaponColor;
  double weaponRange;
  int    weaponDamage;

  double specialRange;
  double specialVelocity;
  double specialSlowdown;
  int    specialArmour;
  
  int    engage;

  public:
  VuxIntruder(double ox, double oy, double shipAngle, int shipCollideFlag,
    int shipRadarColor, ShipData *shipData, Control *cntrl);

  virtual int activate_weapon();
  virtual int activate_special();
  virtual void materialize();
  virtual void calculate_phase();
};

class VuxLimpet : public AnimatedShot {
  double slowdown_factor;
  Ship* anime;

  public:
  VuxLimpet(double ox, double oy, double ov, double slowdown, double orange,
    int oarmour, Ship *oship, SpaceSprite *osprite, int ofcount, int ofsize);

  virtual void calculate();
  virtual void inflict_damage(SpaceObject *other);
};

VuxIntruder::VuxIntruder(double ox, double oy, double shipAngle,
  int shipCollideFlag, int shipRadarColor, ShipData *shipData, Control *cntrl) :
  Ship(ox, oy, shipAngle, shipCollideFlag, shipRadarColor, shipData, cntrl),
  engage(TRUE)

{
  weaponColor  = get_config_int("Weapon", "Color", 0);
  weaponRange  = scale_range(get_config_float("Weapon", "Range", 0));
  weaponDamage = get_config_int("Weapon", "Damage", 0);

  specialRange    = scale_range(get_config_float("Special", "Range", 0));
  specialVelocity = scale_velocity(get_config_float("Special", "Velocity", 0));
  specialSlowdown = get_config_float("Special", "Slowdown", 1);
  specialArmour   = get_config_int("Special", "Armour", 0);
}

int VuxIntruder::activate_weapon()
{
  spaceFrame->addSpaceLine(new Laser(angle,
    pallete_color[weaponColor], weaponRange, weaponDamage, weapon_rate,
    this, 0.0, (h / 2.0)));
  return(TRUE);
}

int VuxIntruder::activate_special()
{
  spaceFrame->addSpaceObject(new VuxLimpet(0.0, -height() / 2.0,
    specialVelocity, specialSlowdown, specialRange, specialArmour, this,
    data->spriteSpecial, 100, 5));
  return(TRUE);
}

VuxLimpet::VuxLimpet(double ox, double oy, double ov, double slowdown,
  double orange, int oarmour, Ship *oship, SpaceSprite *osprite,
  int ofsize, int ofcount) :
  AnimatedShot(ox, oy, 0.0, ov, 0, orange, oarmour, oship, oship, osprite,
    ofcount, ofsize),
  slowdown_factor(slowdown),
  anime(oship)
{
  if((ship->target) && (!ship->target->isAlly(ship)))
    angle = trajectory_angle(ship->target);
  else
    angle = ship->get_angle() - 180.0;

  vx    = v * cos(angle * ANGLE_RATIO);
  vy    = v * sin(angle * ANGLE_RATIO);

  if(!ship->exists())
    state = 0;
}

void VuxLimpet::calculate()
{
	if(!ship->exists()) {
		state = 0;
		return;
		}

	if(ship && (ship->target) && (!ship->target->isAlly(ship))) {
		angle = trajectory_angle(ship->target);
		vx = v * cos(angle * ANGLE_RATIO);
		vy = v * sin(angle * ANGLE_RATIO);
		}
	AnimatedShot::calculate();
	}

void VuxLimpet::inflict_damage(SpaceObject *other) {
	if(!other->isShip()) {
		if (other->damage_factor || other->mass) state = 0;
		return;
		}

	Ship *target = (Ship *) other;

	tw_sample(anime->data->sampleSpecial[1]);

	//MYCODE begin
	int hx,hy,tries, col;

	//find a random spot on the target ship where it "exist" 
	tries = 0;
	while (tries < 30) {
		hx = 18 + (rand() % 27);
		hy = 15 + (rand() % 36);
		tries++;
		col = getpixel(target->panelBitmap[0],hx,hy);
		if (col == bitmap_mask_color(target->panelBitmap[0])) continue;
		if (col == 0) continue;
		if (col == -1) continue;
		if (col == palette_color[8]) continue;
		break;
		} 

	// draw the Limplet on the ship panel
	BITMAP *s = sprite->get_bitmap(sprite_index);
	stretch_sprite(target->panelBitmap[0], s, hx - s->w/4, hy - s->h/4,	s->w/2, s->h/2 );
	target->update_panel = TRUE;
	target->speed_loss += slowdown_factor;
	target->handle_damage(this);
	state = 0;
	}

void VuxIntruder::materialize() {
	Ship::materialize();
	return;
	}

void VuxIntruder::calculate_phase() {
  if (target && (distance(target) > 200)) {
		x = target->normal_x() + (cos(angle * ANGLE_RATIO) * 100.0);
		y = target->normal_y() + (sin(angle * ANGLE_RATIO) * 100.0);
		angle = trajectory_angle(target);
		if (angle > 360) angle -= 360;
		if (angle < 0) angle += 360;
		}
	sprite_index = int(16+angle * 64 / 360.) & 63;
	Ship::calculate_phase();
	return;
	}

Ship *getVuxIntruder(double sX, double sY, double sAngle,
  int sCollideFlag, int sRadarColor, ShipData *sData, Control *cntrl) {
  return(new VuxIntruder(sX, sY, sAngle, sCollideFlag, sRadarColor, sData, cntrl));
}

static ShipRegistration ignore_me(
  "vuxin",
  "Vux Intruder",
  getVuxIntruder
	);
